package pages

const (
	New = "new"

	Event        = "event"
	Classified   = "classified"
	ForSale      = "for-sale"
	ForFree      = "for-free"
	WantToBuy    = "to-buy"
	WantToBorrow = "to-borrow"
	Housing      = "housing"
	ForRent      = "for-rent"
	Request      = "request"
)

var Names = Root{
	Home:               "home",
	CommunityResources: "community-resources",
	Editor:             "editor",
	FAQ:                "faq",
	TermsOfService:     "terms-of-service",
	PrivacyPolicy:      "privacy-policy",
	Donate:             "donate",
	DonateComplete:     "donate-complete",
	Contact:            "contact",

	Help: HelpSection{
		Classified: ClassifiedHelpSection{
			Help:     "help",
			RSS:      help(classified("rss")),
			Search:   help(classified("search")),
			Markdown: help(classified("markdown")),
		},
	},
	OAuth: OauthSection{
		Authorize: "oauth2-authorize",
	},
	User: UserSection{
		Login:         user("login"),
		LoginToken:    user("login-token"),
		Logout:        user("logout"),
		RequireName:   user("require-name"),
		Redirect:      user("redirect"),
		Account:       user("account"),
		DeleteAccount: user("delete-account"),
		Unsubscribe:   user("unsubscribe"),
		JoinDigest:    user("join-digest"),
	},
	Event: EventSection{
		Browse:      Event,
		RSS:         rss(Event),
		One:         one(Event),
		AddCalendar: one(Event) + "-calendar",
		New:         event(New),
		Edit:        edit(Event),
		NewPhotos:   edit(Event) + "-photos",
		Delete:      delete(Event),
	},
	Posts: PostsSection{
		MyPosts:         "my-posts",
		NewPhotos:       "post-new-photos",
		DeletePhoto:     "post-delete-photo",
		Publish:         "post-publish",
		Delist:          "post-delist",
		Relist:          "post-relist",
		Delete:          "post-delete",
		ModeratorDelete: moderator("post-delete"),
	},
	Classified: ClassifiedSection{
		One:   one("classified"),
		New:   classified(New),
		Edit:  edit("classified"),
		Kiosk: classified("kiosk"),

		ForSale: ClassifiedPage{
			Browse: classified(ForSale),
			RSS:    rss(classified(ForSale)),
		},
		ForFree: ClassifiedPage{
			Browse: classified(ForFree),
		},
		WantToBuy: ClassifiedPage{
			Browse: classified(WantToBuy),
		},
		WantToBorrow: ClassifiedPage{
			Browse: classified(WantToBorrow),
		},
	},
	Housing: HousingSection{
		New:    housing(New),
		Browse: Housing,
		RSS:    rss(Housing),
		One:    one(Housing),
		Edit:   edit(Housing),
	},
	Admin: AdminSection{
		Home:        admin("home"),
		Users:       admin("users"),
		UserDetails: admin("user-details"),
		UserDelete:  admin("user-delete"),
		Emails:      admin("emails"),
	},
	Hooks: HooksSection{
		IncomingEmail:  hook("support-email"),
		Deliverability: hook("deliverability"),
	},
	API: APISection{
		Stripe: StripeSection{
			CreateCustomer:      "create-customer",
			CreateSubscription:  "create-subscription",
			CreatePaymentIntent: "create-payment-intent",
		},
		Deliverability: DeliverabilitySection{
			Login: api("deliverability-login"),
		},
	},
}

type OauthSection struct {
	Authorize string
}

type UserSection struct {
	Login         string
	LoginToken    string
	Logout        string
	RequireName   string
	Redirect      string
	Account       string
	DeleteAccount string
	Unsubscribe   string
	JoinDigest    string
}

type EventSection struct {
	Browse      string
	RSS         string
	One         string
	AddCalendar string
	New         string
	Edit        string
	NewPhotos   string
	Delete      string
}

type ClassifiedPage struct {
	Browse string
	RSS    string
}

type PostsSection struct {
	MyPosts         string
	NewPhotos       string
	DeletePhoto     string
	Publish         string
	Delist          string
	Relist          string
	Delete          string
	ModeratorDelete string
}

type ClassifiedSection struct {
	One  string
	New  string
	Edit string

	ForSale      ClassifiedPage
	ForFree      ClassifiedPage
	WantToBuy    ClassifiedPage
	WantToBorrow ClassifiedPage

	Kiosk string
}

type HousingSection struct {
	New    string
	Browse string
	RSS    string
	One    string
	Edit   string
}

type AdminSection struct {
	Home        string
	Users       string
	UserDetails string
	UserDelete  string
	Emails      string
}

type HooksSection struct {
	IncomingEmail  string
	Deliverability string
}

type APISection struct {
	Stripe         StripeSection
	Deliverability DeliverabilitySection
}

type StripeSection struct {
	CreateCustomer      string
	CreateSubscription  string
	CreatePaymentIntent string
}

type DeliverabilitySection struct {
	Login string
}

type HelpSection struct {
	Classified ClassifiedHelpSection
}

type ClassifiedHelpSection struct {
	Help     string
	RSS      string
	Search   string
	Markdown string
}

type Root struct {
	OAuth      OauthSection
	User       UserSection
	Event      EventSection
	Posts      PostsSection
	Classified ClassifiedSection
	Housing    HousingSection
	Admin      AdminSection
	Hooks      HooksSection
	API        APISection
	Help       HelpSection

	Home               string
	CommunityResources string
	Editor             string
	FAQ                string
	TermsOfService     string
	PrivacyPolicy      string
	Donate             string
	DonateComplete     string
	Contact            string
}

func user(pageName string) string {
	return "user-" + pageName
}

func event(pageName string) string {
	return "event-" + pageName
}

func housing(pageName string) string {
	return "housing-" + pageName
}

func classified(pageName string) string {
	return "classified-" + pageName
}

func admin(pageName string) string {
	return "admin-" + pageName
}

func rss(pageName string) string {
	return pageName + "-rss"
}

func hook(hookName string) string {
	return "hook-" + hookName
}

func api(apiName string) string {
	return "api-" + apiName
}

func help(pageName string) string {
	return "help-" + pageName
}

func one(pageName string) string {
	return pageName + "-one"
}

func edit(pageName string) string {
	return pageName + "-edit"
}

func delete(pageName string) string {
	return pageName + "-delete"
}

func moderator(pageName string) string {
	return "moderator-" + pageName
}
