CREATE TABLE IF NOT EXISTS mailing_lists (
    id integer NOT NULL PRIMARY KEY AUTOINCREMENT,
    -- Name must be unique
    name text NOT NULL,
    description text NOT NULL,
    creator integer NOT NULL,
    created_at datetime NOT NULL,

    -- Prefix the message subject with this
    -- This will also be used to construct the email address.
    -- It must be unique.
    short_name text NOT NULL,

    -- Any member of the mailing list may send messages.
    -- If false, then only members with is_sender.
    allow_all_senders bool NOT NULL DEFAULT true,
    
    -- Visible in the mailing list browser
    listed bool NOT NULL DEFAULT true,

    -- Instead of joining, users request to join and must be approved.
    require_approval bool NOT NULL DEFAULT false,

    CONSTRAINT `mailing_list_creator`
        FOREIGN KEY (`creator`)
        REFERENCES `users` (`id`)
        ON DELETE CASCADE
);

CREATE TABLE IF NOT EXISTS mailing_list_users (
    user_id integer NOT NULL,
    mailing_list_id integer NOT NULL,
    created_at datetime NOT NULL,

    -- Owner has all permissions, such as creating moderators.
    is_owner bool NOT NULL DEFAULT false,

    -- Moderators may remove messages, threads, and members.
    is_moderator bool NOT NULL DEFAULT false,

    -- If mailing list has require_approval=true then only 
    -- members with this permission can view/send/receive.
    is_approved bool NOT NULL DEFAULT false,

    -- If mailing list has allow_all_senders=false then only members
    -- with this permission can send to the list.
    is_sender bool NOT NULL,

    -- If false, member will only see messages in the web UI.
    receive_emails bool NOT NULL DEFAULT true, 

    PRIMARY KEY (`user_id`, `mailing_list_id`),

    CONSTRAINT `mailing_list_users_user_id`
        FOREIGN KEY (`user_id`)
        REFERENCES `users` (`id`)
        ON DELETE CASCADE,

    CONSTRAINT `mailing_list_users_mailing_list_id`
        FOREIGN KEY (`mailing_list_id`)
        REFERENCES `mailing_lists` (`id`)
        ON DELETE CASCADE
);

CREATE TABLE IF NOT EXISTS mailing_list_threads (
    id integer NOT NULL PRIMARY KEY AUTOINCREMENT,
    subject text NOT NULL,
    mailing_list_id integer NOT NULL,
    created_at datetime NOT NULL,

    CONSTRAINT `mailing_list_threads_mailing_list_id`
        FOREIGN KEY (`mailing_list_id`)
        REFERENCES `mailing_lists` (`id`)
        ON DELETE CASCADE
);

CREATE TABLE IF NOT EXISTS mailing_list_messages (
    id integer NOT NULL PRIMARY KEY AUTOINCREMENT,
    thread_id integer NOT NULL,
    user_id integer NOT NULL,
    body text NOT NULL,
    created_at datetime NOT NULL,

    -- Only present on messages received by email
    postmark_message_id text,

    CONSTRAINT `mailing_list_messages_user_id`
        FOREIGN KEY (`user_id`)
        REFERENCES `users` (`id`)
        ON DELETE CASCADE,

    CONSTRAINT `mailing_list_messages_thread_id`
        FOREIGN KEY (`thread_id`)
        REFERENCES `mailing_list_threads` (`id`)
        ON DELETE CASCADE
);
