CREATE TABLE IF NOT EXISTS events (
    id integer NOT NULL PRIMARY KEY AUTOINCREMENT,
    title text NOT NULL,
    description text NOT NULL,
    creator integer NOT NULL,
    created_at datetime NOT NULL,
    location text NOT NULL,

    -- NOTE: The data, start_time, and end_time columns have been updated in a later migration

    -- Dates and times are assumed to be in local time, since this application
    -- serves a narrow geographic region by design.

    -- Enforced at create time that date is >= current date
    -- Stored in the format "2023-01-01"
    date text NOT NULL,

    -- Events do not require a time, but if they have an end_time then they must
    -- have a start_time. Having just a start_time is okay.
    start_time text,
    end_time text,

    CONSTRAINT `event_creator`
        FOREIGN KEY (`creator`)
        REFERENCES `users` (`id`)
        ON DELETE CASCADE
);
