

CREATE TABLE IF NOT EXISTS events_prime (
    id integer NOT NULL PRIMARY KEY AUTOINCREMENT,
    title text NOT NULL,
    description text NOT NULL,
    creator integer NOT NULL,
    created_at datetime NOT NULL,
    location text NOT NULL,

    has_time bool NOT NULL DEFAULT false,

    -- Stored in UTC. If has_time is invalid, the time portion should be ignored.
    start_time datetime NOT NULL,

    -- Stored in UTC. If event has no end time, this will be null.
    -- Will be the same date (locally) as the start_time.
    end_time datetime,

    CONSTRAINT `event_creator`
        FOREIGN KEY (`creator`)
        REFERENCES `users` (`id`)
        ON DELETE CASCADE
);

INSERT INTO events_prime 
    (id, title, description, creator, created_at, location, has_time, start_time)
SELECT
    id,
    title,
    description,
    creator,
    created_at,
    location,
    (start_time is not null),
    datetime(date || ' ' || start_time || '-07:00')
FROM events
WHERE end_time is null;

INSERT INTO events_prime 
    (id, title, description, creator, created_at, location, has_time, start_time, end_time)
SELECT
    id,
    title,
    description,
    creator,
    created_at,
    location,
    start_time is not null,
    datetime(date || ' ' || start_time || '-07:00'),
    datetime(date || ' ' || end_time || '-07:00')
FROM events
WHERE end_time is not null;

DROP TABLE events;
ALTER TABLE events_prime RENAME TO events;
