-- name: GetUserByEmail :one
SELECT * FROM users
WHERE email = lower(?)
LIMIT 1;

-- name: GetUserByID :one
SELECT * FROM users
WHERE id = ?
LIMIT 1;

-- name: GetUsers :many
SELECT * FROM users;

-- name: SearchUsers :many
SELECT * FROM users
WHERE
    name LIKE ?
    OR email like ?;

-- name: GetUserInfo :one
SELECT name, email FROM users
WHERE id = ?;

-- name: GetUserCount :one
SELECT COUNT(*) FROM users;

-- name: CreateUser :execlastid
INSERT INTO users (
    name, email, created_at
) VALUES (
    ?, ?, datetime('now')
);

-- name: AddUserToGroup :exec
INSERT INTO user_groups (
    user_id, group_id
) VALUES (
    ?, ?
);

-- name: SetUserVerified :exec
UPDATE users
SET verified = true
WHERE id = ?;

-- name: SetUserName :exec
UPDATE users
SET name = ?
WHERE id = ?;

-- name: SetUserUsername :exec
UPDATE users
SET username = ?
WHERE id = ?;

-- name: SetUserNameIfEmpty :exec
UPDATE users
SET
    name = ?
WHERE email = ? AND name is null;

-- name: SetUserStripeCustomerID :exec
UPDATE users
SET stripe_customer_id = ?
WHERE email = ?;

-- name: DeleteUser :exec
-- DeleteUser removes a user from the database. All dependent tables including
-- posts and photos will cascade.
DELETE FROM users
WHERE id = ?;

-- name: IncrementPostsCreatedCount :exec
-- IncrementPostsCreatedCount increases the counter on the user for number of posts they have created.
UPDATE users
SET
    count_posts_created = count_posts_created + 1
WHERE id = ?;

-- name: IncrementEventsCreatedCount :exec
-- IncrementEventsCreatedCount increases the counter on the user for number of events they have created.
UPDATE users
SET
    count_events_created = count_events_created + 1
WHERE id = ?;

-- name: IncrementLoginCount :exec
UPDATE users
SET
    count_logins = count_logins + 1,
    last_login = datetime('now')
WHERE id = ?;

-- name: IncrementPageLoadCount :exec
UPDATE users
SET
    count_page_loads = count_page_loads + 1
WHERE id = ?;

-- name: IncrementUserEmailResponseCount :exec
UPDATE users
SET
    count_posts_response_emails = count_posts_response_emails + 1
WHERE email = ?;

-- name: IncrementUserEmailOtherCount :exec
UPDATE users
SET
    count_other_incoming_email = count_other_incoming_email + 1
WHERE email = ?;

-- name: SetDigestMarkedSpam :exec
UPDATE users
SET
    digest_marked_spam = true
WHERE email = ?;

-- name: SetAdminMarkedSpam :exec
UPDATE users
SET
    admin_marked_spam = true
WHERE email = ?;

-- name: SetUserMessageMarkedSpam :exec
UPDATE users
SET
    user_message_marked_spam = true
WHERE email = ?;

-- name: IncrementBouncedEmailCount :exec
-- IncrementBouncedEmailCount increments the number of times we have sent this user an email but it has bounced.
UPDATE users
SET
    count_bounced_emails = count_bounced_emails + 1
WHERE email = ?
