package controller

import (
	"net/http"
	"os"
	"testing"

	"github.com/labstack/echo/v4"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
	"go.uber.org/zap"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/context"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/postbase"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/services"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/tests"
)

var c *services.Container

func TestMain(m *testing.M) {
	// Create a new container
	configPath := tests.LoadConfigPath()
	c = services.NewContainer(configPath, zap.NewNop())

	// Run tests
	exitVal := m.Run()

	// Shutdown the container
	if err := c.Shutdown(); err != nil {
		panic(err)
	}

	os.Exit(exitVal)
}

func TestController_Redirect(t *testing.T) {
	c.Web.GET("/path/:first/and/:second", func(c echo.Context) error {
		return nil
	}).Name = "redirect-test"

	ctx, _ := tests.NewContext(c.Web, "/abc")
	ctr := NewController(c)
	err := ctr.Redirect(ctx, "redirect-test", "one", "two")
	require.NoError(t, err)
	assert.Equal(t, "/path/one/and/two", ctx.Response().Header().Get(echo.HeaderLocation))
	assert.Equal(t, http.StatusFound, ctx.Response().Status)
}

func TestController_RedirectToPostBrowser(t *testing.T) {
	// Routes are not registered in this custom controller, so the test fails.
	t.SkipNow()

	ctx, _ := tests.NewContext(c.Web, "/abc/123")
	ctx.Set(context.PostBaseKey, &postbase.PostBase{
		PostsBase: sqlc.PostsBase{
			PostType: constants.Housing,
		},
	})

	ctr := NewController(c)
	err := ctr.RedirectToPostBrowser(ctx)
	require.NoError(t, err)
	assert.Equal(t, "/housing", ctx.Response().Header().Get(echo.HeaderLocation))
	assert.Equal(t, http.StatusFound, ctx.Response().Status)
}

// TODO: Add TestController_RenderHTML
// TODO: Add TestController_RenderXML
