package controller

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/tests"
)

func TestFormSubmission(t *testing.T) {
	type formTest struct {
		Name       string `valid:"required"`
		Email      string `valid:"required,email"`
		Submission FormSubmission
	}

	ctx, _ := tests.NewContext(c.Web, "/")
	form := formTest{
		Name:  "",
		Email: "a@a.com",
	}
	err := form.Submission.Process(ctx, form)
	assert.NoError(t, err)

	assert.True(t, form.Submission.HasErrors())
	assert.True(t, form.Submission.FieldHasErrors("Name"))
	assert.False(t, form.Submission.FieldHasErrors("Email"))
	require.Len(t, form.Submission.GetFieldErrors("Name"), 1)
	assert.Len(t, form.Submission.GetFieldErrors("Email"), 0)
	assert.Equal(t, "This field is required.", form.Submission.GetFieldErrors("Name")[0])
	assert.Equal(t, "border-red-300", form.Submission.GetFieldBorderClass("Name"))
	assert.Equal(t, "border-gray-300", form.Submission.GetFieldBorderClass("Email"))
	assert.False(t, form.Submission.IsDone())
}
