package controller

import (
	"fmt"
	"net/http"
	"testing"

	echomw "github.com/labstack/echo/v4/middleware"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/context"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/tests"
)

func TestNewPage(t *testing.T) {
	ctx, _ := tests.NewContext(c.Web, "/")
	p := NewPage(ctx, c)
	assert.Same(t, ctx, p.Context)
	assert.NotNil(t, p.ToURL)
	assert.Equal(t, "/", p.Path)
	assert.Equal(t, "/", p.URL)
	assert.Equal(t, http.StatusOK, p.StatusCode)
	assert.Equal(t, Pager{}, p.Pager)
	assert.Empty(t, p.Headers)
	assert.False(t, p.IsAuth)
	assert.Empty(t, p.CSRF)
	assert.Empty(t, p.RequestID)

	ctx, _ = tests.NewContext(c.Web, "/abc?def=123")
	usr, err := c.DB.User.CreateMockUser()
	require.NoError(t, err)
	ctx.Set(context.AuthenticatedUserKey, usr)
	ctx.Set(echomw.DefaultCSRFConfig.ContextKey, "csrf")
	p = NewPage(ctx, c)
	assert.Equal(t, "/abc", p.Path)
	assert.Equal(t, "/abc?def=123", p.URL)
	assert.True(t, p.IsAuth)
	assert.Equal(t, usr, p.AuthUser)
	assert.Equal(t, "csrf", p.CSRF)
}

func TestRSSLink(t *testing.T) {
	cases := []struct {
		Input    string
		Expected string
	}{
		{
			Input:    "/one/two",
			Expected: "/one/two.rss",
		},
		{
			Input:    "/one/two?testing",
			Expected: "/one/two.rss?testing",
		},
		{
			Input:    "/one",
			Expected: "/one.rss",
		},
	}

	for i, tc := range cases {
		t.Run(fmt.Sprintf("Test Case %d", i), func(t *testing.T) {
			ctx, _ := tests.NewContext(c.Web, tc.Input)
			p := NewPage(ctx, c)

			result := p.RSSLink()
			assert.Equal(t, tc.Expected, result)
		})
	}
}
