package controller

import (
	"net/url"

	"github.com/labstack/echo/v4"
	"go.uber.org/zap"

	"git.grosinger.net/tgrosinger/tonyslist/config"
	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/context"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/services/mail"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/services/mail/types"
)

type UserController struct {
	Controller
}

// SendLoginEmail sends an email to the user with a login link.
func (c UserController) SendLoginEmail(ctx echo.Context, userID int, email string, isNew bool, query url.Values) error {
	token, err := c.Container.Auth.GenerateJWTEmailAuthToken(email)
	if err != nil {
		return c.Fail(err, "unable to generate password token email")
	}

	path := ctx.Echo().Reverse(pages.Names.User.LoginToken, userID, token)
	path += "?" + query.Encode()

	// In dev, just log the link to the console.
	if c.Container.Config.App.Environment != config.EnvProduction {
		context.GetLogger(ctx).Info("Login link", zap.String("link", path))
	}

	templateData := map[string]interface{}{
		"new_user":      isNew,
		"domain":        c.Container.Config.App.PublicURL,
		"path":          path,
		"support_email": c.Container.Config.Mail.SupportAddress,
	}

	m, err := c.Container.Mail.Template(mail.TemplateArgs{
		MessageStream: types.AccountManagement,
		To:            email,
		Subject:       c.Container.Config.App.Name + " Login",
		HTMLTemplate:  "register_html",
		TextTemplate:  "register_text",
		TemplateData:  templateData,
	})
	if err != nil {
		return c.Fail(err, "unable to send password token email")
	}

	c.Container.Mail.Send(context.GetLogger(ctx), []*types.Mail{m}, nil)
	return nil
}
