package cron

import (
	"context"
	"time"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/generic"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/services"
	"go.uber.org/zap"
)

// TODO: Cleanup events that are more than 1 month in the past

func RegisterDefaultCronJobs(c *services.CronRunner) {
	c.RegisterMetricsJobs(
		reportUserCount,
		reportClassifiedPostCount,
		reportHousingPostCount,
		reportDatabaseStats)
	c.RegisterHourlyJobs(emailExpiredPostOwners, sendDigestEmails)
	c.RegisterDailyJobs(deleteExpiredPosts)
}

type lastRun struct {
	now     time.Time
	lastRun time.Time
}

// getLastRunTime returns the time that a cron job last executed and updates the
// time that it last run to now.
func getLastRunTime(logger *zap.Logger, db *models.Client, name string) (lastRun, error) {
	lr := lastRun{
		now: time.Now(),
	}

	// I was having trouble with this table being locked. I think this is
	// because multiple cron jobs are run in parallel and both are attempting to
	// use this table.
	retry := generic.QuickLinearRetry()

	logger1 := logger.With(zap.String("action", "GetCronLastRun"))
	err := retry.Retry(logger1, func() error {
		var err error
		lr.lastRun, err = db.C.GetCronLastRun(context.Background(), name)
		return err
	})
	if err != nil {
		return lr, err
	}

	logger2 := logger.With(zap.String("action", "SetCronLastRun"))
	err = retry.Retry(logger2, func() error {
		return db.C.SetCronLastRun(context.Background(), sqlc.SetCronLastRunParams{
			Name:    name,
			LastRun: lr.now,
		})
	})

	return lr, err
}
