package cron

import (
	"context"
	"time"

	"go.uber.org/zap"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/services"
)

// deleteExpiredPosts completely removes posts from the database 3 months after
// their expiration or 1 month after being explicitly delisted.
func deleteExpiredPosts(c *services.Container) {
	now := time.Now()
	expiredPublishedAt := now.Add(-1 * constants.ClassifiedsDuration)

	oneMonthBeforeExpired := expiredPublishedAt.AddDate(0, -1, 0)
	threeMonthsBeforeExpired := expiredPublishedAt.AddDate(0, -3, 0)

	c.Logger.Info("cron cleaning up expired posts published before date",
		zap.Time("published_at", threeMonthsBeforeExpired))
	err := c.DB.C.DeletePostsExpiredBefore(context.Background(), threeMonthsBeforeExpired)
	if err != nil {
		c.Logger.Error("cron failed to delete old expired posts", zap.Error(err))
	}

	c.Logger.Info("cron cleaning up unlisted posts published before date",
		zap.Time("published_at", oneMonthBeforeExpired))
	err = c.DB.C.DeletePostsUnlistedBefore(context.Background(), oneMonthBeforeExpired)
	if err != nil {
		c.Logger.Error("cron failed to delete old unlisted posts", zap.Error(err))
	}
}
