package cron

import (
	"context"

	"github.com/prometheus/client_golang/prometheus"
	"go.uber.org/zap"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/metrics"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/services"
)

func reportUserCount(c *services.Container) {
	userCount, err := c.DB.C.GetUserCount(context.Background())
	if err != nil {
		c.Logger.Error("unable to retrieve user count in cron",
			zap.Error(err))
	}

	metrics.UserCount.Set(float64(userCount))
}

func reportClassifiedPostCount(c *services.Container) {
	postCounts, err := c.DB.C.GetActiveClassifiedPostsCount(context.Background())
	if err != nil {
		c.Logger.Error("unable to retrieve classified post counts in cron",
			zap.Error(err))
	}

	for _, row := range postCounts {
		metrics.ActiveClassifieds.With(prometheus.Labels{
			metrics.LabelPostType: string(row.ClassifiedType),
		}).Set(float64(row.Count))
	}
}

func reportHousingPostCount(c *services.Container) {
	count, err := c.DB.C.GetActiveHousingPostsCount(context.Background())
	if err != nil {
		c.Logger.Error("unable to retrieve housing for rent post counts in cron",
			zap.Error(err))
	} else {
		metrics.ActiveHousing.With(prometheus.Labels{
			metrics.LabelPostType: string(constants.Housing),
		}).Set(float64(count))
	}
}

func reportDatabaseStats(c *services.Container) {
	stats, err := c.DB.C.GetAllStats(context.Background())
	if err != nil {
		c.Logger.Error("unable to retrieve stats in cron", zap.Error(err))
		return
	}
	for _, stat := range stats {
		metrics.StatsGauge.With(prometheus.Labels{
			metrics.LabelName: string(stat.Name),
		}).Set(float64(stat.Value))
	}
}
