package generic

import "golang.org/x/exp/constraints"

// Map turns a slice of one type into a slice of another type via a mapping
// function.
func Map[T any, M any](a []T, f func(T) M) []M {
	n := make([]M, len(a))
	for i, e := range a {
		n[i] = f(e)
	}
	return n
}

// TODO: Replace this with slices.DeleteFunc after upgrading to Go 1.21
// https://pkg.go.dev/slices@master#DeleteFunc
func Filter[T any](a []T, f func(T) bool) []T {
	n := make([]T, 0, len(a))
	for _, e := range a {
		if f(e) {
			n = append(n, e)
		}
	}
	return n
}

func Min[T constraints.Ordered](a, b T) T {
	if a < b {
		return a
	}
	return b
}

func Max[T constraints.Ordered](a, b T) T {
	if a < b {
		return b
	}
	return a
}

// Batches turns a list into a list of lists of size batchSize.
func Batches[T any](a []T, batchSize int) [][]T {
	batches := make([][]T, 0, len(a)/batchSize+1)

	for i := 0; i < len(a); i += batchSize {
		j := i + batchSize
		if j >= len(a) {
			j = len(a)
		}

		batches = append(batches, a[i:j])
	}
	return batches
}
