package generic

import (
	"strconv"
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestMap(t *testing.T) {
	input := []int{1, 2, 3, 4}
	output := Map(input, func(i int) string {
		return strconv.Itoa(i)
	})

	assert.Equal(t, []string{"1", "2", "3", "4"}, output)
}

func TestBatchs(t *testing.T) {
	cases := []struct {
		name      string
		input     []int
		batchSize int
		expected  [][]int
	}{
		{
			name:      "empty",
			input:     []int{},
			batchSize: 500,
			expected:  [][]int{},
		},
		{
			name:      "one element",
			input:     []int{1},
			batchSize: 500,
			expected:  [][]int{{1}},
		},
		{
			name:      "two full batches",
			input:     []int{1, 2, 3, 4},
			batchSize: 2,
			expected:  [][]int{{1, 2}, {3, 4}},
		},
		{
			name:      "partial final batch",
			input:     []int{1, 2, 3, 4, 5},
			batchSize: 3,
			expected:  [][]int{{1, 2, 3}, {4, 5}},
		},
	}

	for _, tc := range cases {
		t.Run(tc.name, func(t *testing.T) {
			result := Batches(tc.input, tc.batchSize)
			assert.Equal(t, tc.expected, result)
		})
	}
}
