package generic

import (
	"time"

	"go.uber.org/zap"
)

// NOTE: This probably isn't the right home for this file.

type delayType int

const (
	constant delayType = iota
	linear
	exponential
)

func QuickLinearRetry() Backoff {
	return Backoff{
		attempts:  5,
		delayType: linear,
		delay:     time.Millisecond * 500,
	}
}

type Backoff struct {
	attempts    int
	delayType   delayType
	delay       time.Duration
	delayFactor float64
	maxDelay    time.Duration
}

func (b Backoff) WithConstantDelay(delay time.Duration) Backoff {
	b.delayType = constant
	b.delay = delay
	return b
}

func (b Backoff) WithLinearDelay(delay time.Duration) Backoff {
	b.delayType = linear
	b.delay = delay
	return b
}

func (b Backoff) WithExponentialDelay(delay time.Duration, factor float64) Backoff {
	b.delayType = exponential
	b.delay = delay
	b.delayFactor = factor
	return b
}

func (b Backoff) WithMaxDelay(delay time.Duration) Backoff {
	b.maxDelay = delay
	return b
}

func (b Backoff) Retry(logger *zap.Logger, fn func() error) error {
	currentDelay := b.delay
	var err error
	for i := 0; i < b.attempts; i++ {
		err = fn()
		if err == nil {
			return nil
		}

		if logger != nil {
			logger.Warn("Retry attempt failed", zap.Int("attempt", i), zap.Error(err))
			logger.Info("Delaying before retry", zap.Duration("delay", currentDelay))
		}
		time.Sleep(currentDelay)

		switch b.delayType {
		case linear:
			currentDelay += b.delay
		case exponential:
			currentDelay *= time.Duration(b.delayFactor)
		}

		if currentDelay > b.maxDelay {
			currentDelay = b.maxDelay
		}
	}
	return err
}
