package metrics

import (
	"github.com/prometheus/client_golang/prometheus"
	"github.com/prometheus/client_golang/prometheus/promauto"

	"git.grosinger.net/tgrosinger/tonyslist/config"
)

const (
	MetricDonateCompleteUnsuccessful = "complete_unsuccessful"
	MetricDonatePaymentIntent        = "payment_intent"
	MetricDonateCreateSubscription   = "create_subscription"
)

var DonateErrorCount *prometheus.CounterVec

func initializeDonateMetrics(cfg *config.Config) {
	DonateErrorCount = promauto.NewCounterVec(
		prometheus.CounterOpts{
			Namespace: cfg.App.Name,
			Subsystem: "donate",
			Name:      "error_count",
			Help:      "Number of errors encountered while processing donations",
		},
		[]string{LabelError},
	)

	// Initialize with all possible label keys at 0 so that alerts work properly.
	DonateErrorCount.WithLabelValues(MetricDonateCompleteUnsuccessful).Add(0)
	DonateErrorCount.WithLabelValues(MetricDonatePaymentIntent).Add(0)
	DonateErrorCount.WithLabelValues(MetricDonateCreateSubscription).Add(0)
}
