package metrics

import (
	"git.grosinger.net/tgrosinger/tonyslist/config"
	"github.com/prometheus/client_golang/prometheus"
	"github.com/prometheus/client_golang/prometheus/promauto"
)

var (
	BounceCount            prometheus.CounterVec
	SpamCount              prometheus.CounterVec
	IncomingEmail          prometheus.CounterVec
	IncomingEmailUnhandled prometheus.CounterVec

	LabelMessageStream = "message_stream"
	LabelEventType     = "event_type"

	// LabelID refers to the ID of the object type corresponding with the
	// message stream. For example, it will be a posts_base ID for the
	// classified email forwarding, or an event for RSVPs.
	LabelID = "id"
)

func initializeHookMetrics(cfg *config.Config) {
	BounceCount = *promauto.NewCounterVec(
		prometheus.CounterOpts{
			Namespace: cfg.App.Name,
			Subsystem: "hooks",
			Name:      "email_bounce_count",
			Help:      "Number of email bounce reports",
		},
		[]string{LabelMessageStream, LabelEventType},
	)

	SpamCount = *promauto.NewCounterVec(
		prometheus.CounterOpts{
			Namespace: cfg.App.Name,
			Subsystem: "hooks",
			Name:      "email_spam_report_count",
			Help:      "Number of spam reports",
		},
		[]string{LabelMessageStream},
	)

	IncomingEmail = *promauto.NewCounterVec(
		prometheus.CounterOpts{
			Namespace: cfg.App.Name,
			Subsystem: "hooks",
			Name:      "incoming_email_count",
			Help:      "Number of incoming emails handled",
		},
		[]string{LabelMessageStream, LabelID},
	)

	IncomingEmailUnhandled = *promauto.NewCounterVec(
		prometheus.CounterOpts{
			Namespace: cfg.App.Name,
			Subsystem: "hooks",
			Name:      "incoming_email_unhandled_count",
			Help:      "Number of incoming emails which could not be handled",
		},
		[]string{LabelError},
	)
}
