package metrics

import (
	"github.com/prometheus/client_golang/prometheus"
	"github.com/prometheus/client_golang/prometheus/promauto"

	"git.grosinger.net/tgrosinger/tonyslist/config"
)

var (
	HTTPQRRequestCount prometheus.Counter
	HTTPRequestCount   *prometheus.CounterVec
	HTTPRequestTiming  *prometheus.HistogramVec

	LabelMethod     = "method"
	LabelRouterPath = "router_path"
	LabelRealPath   = "real_path"
	LabelCode       = "code"
)

func initializeHttpMetrics(cfg *config.Config) {
	HTTPQRRequestCount = promauto.NewCounter(
		prometheus.CounterOpts{
			Namespace: cfg.App.Name,
			Subsystem: "http",
			Name:      "qr_requests_count",
			Help:      "Number of http requests from qr code scans",
		},
	)

	HTTPRequestCount = promauto.NewCounterVec(
		prometheus.CounterOpts{
			Namespace: cfg.App.Name,
			Subsystem: "http",
			Name:      "requests_count",
			Help:      "Number of http requests",
		},
		[]string{
			LabelMethod,
			LabelRouterPath,
			LabelRealPath,
			LabelCode,
		},
	)

	HTTPRequestTiming = promauto.NewHistogramVec(
		prometheus.HistogramOpts{
			Namespace: cfg.App.Name,
			Subsystem: "http",
			Name:      "requests_ms_timing",
			Help:      "Duration of requests in milliseconds",
			Buckets:   []float64{.1, .5, 1, 5, 10, 15, 20, 25},
		},
		[]string{
			LabelMethod,
			LabelRouterPath,
			LabelRealPath,
			LabelCode,
		},
	)
}
