package metrics

import (
	"context"
	"fmt"
	"net/http"
	"time"

	"github.com/prometheus/client_golang/prometheus/promhttp"

	"git.grosinger.net/tgrosinger/tonyslist/config"
)

var LabelError = "error"

type Server struct {
	httpServer http.Server
}

func NewMetricsServer(cfg *config.Config) *Server {
	initialize(cfg)

	metricsMux := http.NewServeMux()
	metricsMux.Handle("/metrics", promhttp.Handler())

	return &Server{
		httpServer: http.Server{
			ReadTimeout: 20 * time.Second,
			Addr:        fmt.Sprintf(":%d", cfg.App.MetricsPort),
			Handler:     metricsMux,
		},
	}
}

func (s *Server) Run() error {
	return s.httpServer.ListenAndServe()
}

func (s *Server) Shutdown(ctx context.Context) error {
	return s.httpServer.Shutdown(ctx)
}

func initialize(cfg *config.Config) {
	initializeHttpMetrics(cfg)
	initializeTasksMetrics(cfg)
	initializeMailMetrics(cfg)
	initializePostsMetrics(cfg)
	initializeCronMetrics(cfg)
	initializeStatsMetrics(cfg)
	initializeHookMetrics(cfg)
	initializeDonateMetrics(cfg)
}
