package metrics

import (
	"github.com/prometheus/client_golang/prometheus"
	"github.com/prometheus/client_golang/prometheus/promauto"

	"git.grosinger.net/tgrosinger/tonyslist/config"
)

var (
	TasksImageResizeFailureCount *prometheus.CounterVec
	TasksImageResizeTiming       prometheus.Histogram
)

func initializeTasksMetrics(cfg *config.Config) {
	TasksImageResizeFailureCount = promauto.NewCounterVec(
		prometheus.CounterOpts{
			Namespace: cfg.App.Name,
			Subsystem: "tasks",
			Name:      "image_resize_err_count",
			Help:      "Number of errors while resizing images",
		},
		[]string{
			LabelError,
		},
	)

	TasksImageResizeTiming = prometheus.NewHistogram(
		prometheus.HistogramOpts{
			Namespace: cfg.App.Name,
			Subsystem: "tasks",
			Name:      "image_resize_ms_timing",
			Help:      "Duration of image resize in milliseconds",
			Buckets:   []float64{1, 5, 10, 50, 100, 1000},
		},
	)
}
