package middleware

import (
	"strconv"
	"time"

	"github.com/labstack/echo/v4"
	"github.com/prometheus/client_golang/prometheus"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/metrics"
)

func RequestMetrics() echo.MiddlewareFunc {
	return func(next echo.HandlerFunc) echo.HandlerFunc {
		return func(c echo.Context) error {
			start := time.Now()
			err := next(c)
			duration := time.Since(start)

			labels := prometheus.Labels{
				metrics.LabelMethod:     c.Request().Method,
				metrics.LabelRouterPath: c.Path(),
				metrics.LabelRealPath:   c.Request().URL.Path,
				metrics.LabelCode:       strconv.Itoa(c.Response().Status),
			}
			metrics.HTTPRequestTiming.With(labels).Observe(float64(duration.Milliseconds()))
			metrics.HTTPRequestCount.With(labels).Inc()

			if c.Request().URL.Query().Get("qr") == "t" {
				metrics.HTTPQRRequestCount.Inc()
			}

			return err
		}
	}
}
