package middleware

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"go.uber.org/zap"

	"git.grosinger.net/tgrosinger/tonyslist/config"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/context"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/conv"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/services/stripe"
)

func WithStripeCustomer(s stripe.Client, db *models.Client, cfg config.StripeConfig) echo.MiddlewareFunc {
	return func(next echo.HandlerFunc) echo.HandlerFunc {
		return func(c echo.Context) error {
			user, ok := c.Get(context.AuthenticatedUserKey).(sqlc.User)
			if ok {
				if user.StripeCustomerID.Valid {
					c.Set(context.StripeCustomerIDKey, user.StripeCustomerID.String)
					return next(c)
				}

				customerID, err := s.GetStripeCustomerIDForEmail(c.Request().Context(), user.Email, user.Name)
				if err != nil {
					context.GetLogger(c).Warn("unable to create new customer ID for user",
						zap.Error(err))
					return echo.NewHTTPError(
						http.StatusInternalServerError, "unable to create new customer ID for user")
				}

				user.StripeCustomerID = conv.StringToNullString(customerID)
				c.Set(context.AuthenticatedUserKey, user)
				c.Set(context.StripeCustomerIDKey, customerID)
			}

			return next(c)
		}
	}
}
