package classified

import (
	"database/sql"
	"errors"
	"strconv"

	sq "github.com/Masterminds/squirrel"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/postbase"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
)

type Post struct {
	*postbase.PostBase
	Price sql.NullFloat64 `db:"price"`
}

type SelectInput struct {
	postbase.SelectInput
	PostType constants.ClassifiedType
}

type NewPostInput struct {
	sqlc.CreatePostParams
	ClassifiedType constants.ClassifiedType
	Price          sql.NullFloat64
}

func (i *NewPostInput) SetBuilder(b *postbase.QueryBuilder) error {
	b.Add(ClassifiedType, i.ClassifiedType)

	if i.Price.Valid {
		b.Add(Price, i.Price.Float64)
	}

	return nil
}

type UpdatePostInput struct {
	sqlc.UpdatePostParams
	Price string // float64
}

func (i *UpdatePostInput) SetBuilder(b sq.UpdateBuilder) (sq.UpdateBuilder, error) {
	if i.Price != "" {
		price, err := strconv.ParseFloat(i.Price, 64)
		if err != nil {
			return b, errors.New("unable to parse price into numeric value")
		}
		b = b.Set(Price, price)
	} else {
		b = b.Set(Price, nil)
	}

	return b, nil
}
