package models

import (
	"os"
	"testing"

	"git.grosinger.net/tgrosinger/tonyslist/config"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/tests"
	"go.uber.org/zap"
)

var (
	usr sqlc.User
	db  *Client
)

func TestMain(m *testing.M) {
	logger, _ := zap.NewDevelopment()
	defer logger.Sync()

	configPath := tests.LoadConfigPath()
	cfg, err := config.GetConfig(configPath)
	if err != nil {
		panic(err)
	}

	// Create the database
	db, err = New(logger, &cfg)
	if err != nil {
		panic(err)
	}

	// Create a test user
	if usr, err = db.User.CreateMockUser(); err != nil {
		panic(err)
	}

	// Run tests
	exitVal := m.Run()

	// Shutdown the database
	if err = db.Close(); err != nil {
		panic(err)
	}

	os.Exit(exitVal)
}
