package models

import (
	"context"
	"database/sql"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/conv"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/generic"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/errors"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/postbase"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
)

// PostBase contains operations that can be performed on any type of
// post. Either ForSale/ForRent/etc. or HousingRequest/HousingForRent.
type PostBase struct {
	dbConfig
}

func (c *PostBase) GetOne(ctx context.Context, id int, includeExpired bool) (*postbase.PostBase, error) {
	row, err := c.parent.C.GetPost(ctx, id)
	if err != nil {
		return nil, err
	}
	result := postbase.PostBase{
		PostsBase:   row.PostsBase,
		PhotosCount: row.CountPhotos,
	}

	if !includeExpired && result.IsExpired(constants.ClassifiedsDuration) {
		// TODO: Return a different error if expired and render differently in the UI.
		// (Be sure to look for that error in the handlers)
		return nil, errors.ErrNotFound
	}

	return &result, nil
}

func (c *PostBase) GetMyPosts(ctx context.Context, arg sqlc.GetMyPostsParams) ([]postbase.PostBase, error) {
	results, err := c.parent.C.GetMyPosts(ctx, arg)
	if err != nil {
		return nil, err
	}

	converter := func(r sqlc.PostsBase) postbase.PostBase {
		return postbase.PostBase{PostsBase: r}
	}
	return generic.Map(results, converter), nil
}

func (c *PostBase) CreateAnonymousEmail(valid bool) sql.NullString {
	if !valid {
		return sql.NullString{Valid: false}
	}

	str := conv.RandomNullString(constants.AnonymousEmailLen)
	str.String = constants.AnonymousEmailPrefix + str.String

	return str
}

func (c *PostBase) GetFeaturedPhotos(ctx context.Context, posts []postbase.PostBase) ([]sqlc.Photo, error) {
	photoIDs := make([]int, 0, len(posts))
	for _, post := range posts {
		if post.FeaturedPhoto.Valid {
			photoIDs = append(photoIDs, post.FeaturedPhoto.Int)
		}
	}

	return c.parent.C.GetPhotosByID(ctx, photoIDs)
}
