package models

import (
	"context"
	"database/sql"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestGetModeratorDelistedPost(t *testing.T) {
	postID := createClassifiedPost(t, "Test 1", "Test 1", true)

	postBase, err := db.PostBase.GetOne(context.Background(), postID, true)
	require.NoError(t, err)
	require.NotNil(t, postBase)
	postBase, err = db.PostBase.GetOne(context.Background(), postID, false)
	require.NoError(t, err)
	require.NotNil(t, postBase)

	err = db.C.SetPostModeratorDelisted(context.Background(), postID)
	require.NoError(t, err)

	postBase, err = db.PostBase.GetOne(context.Background(), postID, true)
	require.Equal(t, sql.ErrNoRows, err)
	require.Nil(t, postBase)
}

func TestDeletePost(t *testing.T) {
	id := createClassifiedPost(t, "title cats", "description dogs", true)
	err := db.C.DeletePost(context.Background(), id)
	assert.NoError(t, err)

	// Check that it no longer exists
	_, err = db.PostBase.GetOne(context.Background(), id, true)
	assert.Equal(t, sql.ErrNoRows, err)
}

func TestListPost(t *testing.T) {
	id := createClassifiedPost(t, "title cats", "description dogs", true)

	err := db.C.SetPostUnlisted(context.Background(), id)
	assert.NoError(t, err)

	p, err := db.PostBase.GetOne(context.Background(), id, false)
	assert.NoError(t, err)
	assert.False(t, p.Listed)

	err = db.C.RelistPost(context.Background(), id)
	assert.NoError(t, err)

	p, err = db.PostBase.GetOne(context.Background(), id, false)
	assert.NoError(t, err)
	assert.True(t, p.Listed)
	assert.Less(t, float64(time.Until(p.PublishedAt.Time)*-1), float64(time.Second))

	// Cleanup after ourselves
	err = db.C.DeletePost(context.Background(), id)
	assert.NoError(t, err)
}
