package postbase

import (
	"database/sql"
	"testing"
	"time"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	"github.com/stretchr/testify/assert"
)

func TestIsExpired(t *testing.T) {
	testCases := []struct {
		Name     string
		Input    PostBase
		TTL      time.Duration
		Expected bool
	}{
		{
			Name: "Expired long ago",
			Input: PostBase{
				PostsBase: sqlc.PostsBase{
					PublishedAt: sql.NullTime{
						Valid: true,
						Time:  time.Now().Add(-10 * 24 * time.Hour),
					},
				},
			},
			TTL:      5 * 24 * time.Hour,
			Expected: true,
		},
		{
			Name: "Expired recently",
			Input: PostBase{
				PostsBase: sqlc.PostsBase{
					PublishedAt: sql.NullTime{
						Valid: true,
						Time:  time.Now().Add((-5 * 24 * time.Hour) - time.Minute),
					},
				},
			},
			TTL:      5 * 24 * time.Hour,
			Expected: true,
		},
		{
			Name: "Valid",
			Input: PostBase{
				PostsBase: sqlc.PostsBase{
					PublishedAt: sql.NullTime{
						Valid: true,
						Time:  time.Now().Add(-4 * 24 * time.Hour),
					},
				},
			},
			TTL:      5 * 24 * time.Hour,
			Expected: false,
		},
	}

	for _, tc := range testCases {
		t.Run(tc.Name, func(t *testing.T) {
			result := tc.Input.IsExpired(tc.TTL)
			assert.Equal(t, tc.Expected, result)
		})
	}
}
