// Code generated by sqlc. DO NOT EDIT.
// versions:
//   sqlc v1.27.0
// source: events.sql

package sqlc

import (
	"context"
	"database/sql"
	"time"
)

const createEvent = `-- name: CreateEvent :execlastid
INSERT INTO events
    (title, description, creator, location, has_time, start_time, end_time, created_at)
VALUES
    (?, ?, ?, ?, ?, ?, ?, datetime('now'))
`

type CreateEventParams struct {
	Title       string       `db:"title"`
	Description string       `db:"description"`
	Creator     int          `db:"creator"`
	Location    string       `db:"location"`
	HasTime     bool         `db:"has_time"`
	StartTime   time.Time    `db:"start_time"`
	EndTime     sql.NullTime `db:"end_time"`
}

func (q *Queries) CreateEvent(ctx context.Context, arg CreateEventParams) (int64, error) {
	result, err := q.db.ExecContext(ctx, createEvent,
		arg.Title,
		arg.Description,
		arg.Creator,
		arg.Location,
		arg.HasTime,
		arg.StartTime,
		arg.EndTime,
	)
	if err != nil {
		return 0, err
	}
	return result.LastInsertId()
}

const deleteEvent = `-- name: DeleteEvent :exec
DELETE FROM events
WHERE id = ?
`

// DeleteEvent completely removes an event.
func (q *Queries) DeleteEvent(ctx context.Context, id int) error {
	_, err := q.db.ExecContext(ctx, deleteEvent, id)
	return err
}

const deleteEventPhoto = `-- name: DeleteEventPhoto :exec
DELETE FROM event_photos
WHERE event_id = ?
`

func (q *Queries) DeleteEventPhoto(ctx context.Context, eventID int) error {
	_, err := q.db.ExecContext(ctx, deleteEventPhoto, eventID)
	return err
}

const getEvent = `-- name: GetEvent :one
SELECT id, title, description, creator, created_at, location, has_time, start_time, end_time FROM events
WHERE id = ?
`

func (q *Queries) GetEvent(ctx context.Context, id int) (Event, error) {
	row := q.db.QueryRowContext(ctx, getEvent, id)
	var i Event
	err := row.Scan(
		&i.ID,
		&i.Title,
		&i.Description,
		&i.Creator,
		&i.CreatedAt,
		&i.Location,
		&i.HasTime,
		&i.StartTime,
		&i.EndTime,
	)
	return i, err
}

const getEventPhoto = `-- name: GetEventPhoto :one
SELECT id, event_id, created_at, preview_jpg FROM event_photos
WHERE event_id = ?
LIMIT 1
`

func (q *Queries) GetEventPhoto(ctx context.Context, eventID int) (EventPhoto, error) {
	row := q.db.QueryRowContext(ctx, getEventPhoto, eventID)
	var i EventPhoto
	err := row.Scan(
		&i.ID,
		&i.EventID,
		&i.CreatedAt,
		&i.PreviewJpg,
	)
	return i, err
}

const getRecentlyCreatedEvents = `-- name: GetRecentlyCreatedEvents :many
SELECT id, title, description, creator, created_at, location, has_time, start_time, end_time FROM events
ORDER BY date(created_at) DESC
LIMIT 10
`

func (q *Queries) GetRecentlyCreatedEvents(ctx context.Context) ([]Event, error) {
	rows, err := q.db.QueryContext(ctx, getRecentlyCreatedEvents)
	if err != nil {
		return nil, err
	}
	defer rows.Close()
	var items []Event
	for rows.Next() {
		var i Event
		if err := rows.Scan(
			&i.ID,
			&i.Title,
			&i.Description,
			&i.Creator,
			&i.CreatedAt,
			&i.Location,
			&i.HasTime,
			&i.StartTime,
			&i.EndTime,
		); err != nil {
			return nil, err
		}
		items = append(items, i)
	}
	if err := rows.Close(); err != nil {
		return nil, err
	}
	if err := rows.Err(); err != nil {
		return nil, err
	}
	return items, nil
}

const getUpcomingEventsInRange = `-- name: GetUpcomingEventsInRange :many
SELECT id, title, description, creator, created_at, location, has_time, start_time, end_time FROM events
WHERE 
    date(start_time) >= date('now', 'localtime', 'start of day')
    AND date(start_time, 'localtime') >= date(?, 'start of day')
    AND date(start_time, 'localtime') <= date(?, 'start of day')
ORDER BY start_time ASC
`

type GetUpcomingEventsInRangeParams struct {
	Date   interface{} `db:"date"`
	Date_2 interface{} `db:"date_2"`
}

func (q *Queries) GetUpcomingEventsInRange(ctx context.Context, arg GetUpcomingEventsInRangeParams) ([]Event, error) {
	rows, err := q.db.QueryContext(ctx, getUpcomingEventsInRange, arg.Date, arg.Date_2)
	if err != nil {
		return nil, err
	}
	defer rows.Close()
	var items []Event
	for rows.Next() {
		var i Event
		if err := rows.Scan(
			&i.ID,
			&i.Title,
			&i.Description,
			&i.Creator,
			&i.CreatedAt,
			&i.Location,
			&i.HasTime,
			&i.StartTime,
			&i.EndTime,
		); err != nil {
			return nil, err
		}
		items = append(items, i)
	}
	if err := rows.Close(); err != nil {
		return nil, err
	}
	if err := rows.Err(); err != nil {
		return nil, err
	}
	return items, nil
}

const setEventPhoto = `-- name: SetEventPhoto :exec
INSERT INTO event_photos
    (event_id, created_at, preview_jpg)
VALUES
    (?, datetime('now'), ?)
`

type SetEventPhotoParams struct {
	EventID    int    `db:"event_id"`
	PreviewJpg []byte `db:"preview_jpg"`
}

func (q *Queries) SetEventPhoto(ctx context.Context, arg SetEventPhotoParams) error {
	_, err := q.db.ExecContext(ctx, setEventPhoto, arg.EventID, arg.PreviewJpg)
	return err
}

const updateEvent = `-- name: UpdateEvent :exec
UPDATE events SET
    title = ?,
    description = ?,
    location = ?,
    has_time = ?,
    start_time = ?,
    end_time = ?
WHERE id = ?
`

type UpdateEventParams struct {
	Title       string       `db:"title"`
	Description string       `db:"description"`
	Location    string       `db:"location"`
	HasTime     bool         `db:"has_time"`
	StartTime   time.Time    `db:"start_time"`
	EndTime     sql.NullTime `db:"end_time"`
	ID          int          `db:"id"`
}

func (q *Queries) UpdateEvent(ctx context.Context, arg UpdateEventParams) error {
	_, err := q.db.ExecContext(ctx, updateEvent,
		arg.Title,
		arg.Description,
		arg.Location,
		arg.HasTime,
		arg.StartTime,
		arg.EndTime,
		arg.ID,
	)
	return err
}
