// Code generated by sqlc. DO NOT EDIT.
// versions:
//   sqlc v1.27.0
// source: posts_base.sql

package sqlc

import (
	"context"
	"database/sql"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/conv"
)

const createPost = `-- name: CreatePost :execlastid
INSERT INTO posts_base (
    post_type, title, description, anonymous_email, creator, phone_call, phone_text, created_at, listed
) VALUES (
    ?, ?, ?, ?, ?, ?, ?, datetime('now'), true
)
`

type CreatePostParams struct {
	PostType       constants.PostType `db:"post_type"`
	Title          string             `db:"title"`
	Description    string             `db:"description"`
	AnonymousEmail sql.NullString     `db:"anonymous_email"`
	Creator        int                `db:"creator"`
	PhoneCall      sql.NullString     `db:"phone_call"`
	PhoneText      sql.NullString     `db:"phone_text"`
}

// CreatePost makes a posts_base object which can then be referenced by other
// post tables.
func (q *Queries) CreatePost(ctx context.Context, arg CreatePostParams) (int64, error) {
	result, err := q.db.ExecContext(ctx, createPost,
		arg.PostType,
		arg.Title,
		arg.Description,
		arg.AnonymousEmail,
		arg.Creator,
		arg.PhoneCall,
		arg.PhoneText,
	)
	if err != nil {
		return 0, err
	}
	return result.LastInsertId()
}

const deletePost = `-- name: DeletePost :exec
DELETE FROM posts_base
WHERE id = ?
`

// DeletePost completely removes a post, cascading to other tables.
func (q *Queries) DeletePost(ctx context.Context, id int) error {
	_, err := q.db.ExecContext(ctx, deletePost, id)
	return err
}

const deletePostsExpiredBefore = `-- name: DeletePostsExpiredBefore :exec
DELETE FROM posts_base
WHERE listed AND date(published_at) < date(?)
`

// DeletePostsExpiredBefore completely removes posts which are listed, but which
// have a published_at date before the provided date.
func (q *Queries) DeletePostsExpiredBefore(ctx context.Context, date interface{}) error {
	_, err := q.db.ExecContext(ctx, deletePostsExpiredBefore, date)
	return err
}

const deletePostsUnlistedBefore = `-- name: DeletePostsUnlistedBefore :exec
DELETE FROM posts_base
WHERE NOT listed AND date(published_at) < date(?)
`

// DeletePostsUnlistedBefore completely removes posts which are unlisted and
// which have a published_at date before the provided date.
func (q *Queries) DeletePostsUnlistedBefore(ctx context.Context, date interface{}) error {
	_, err := q.db.ExecContext(ctx, deletePostsUnlistedBefore, date)
	return err
}

const getActivePostsForUserCount = `-- name: GetActivePostsForUserCount :one
SELECT COUNT(*) FROM posts_base pb
WHERE moderator_delisted = false
    AND listed = true
    AND datetime(published_at) > datetime('now', '-14 days')
    AND creator = ?
`

func (q *Queries) GetActivePostsForUserCount(ctx context.Context, creator int) (int, error) {
	row := q.db.QueryRowContext(ctx, getActivePostsForUserCount, creator)
	var count int
	err := row.Scan(&count)
	return count, err
}

const getMyPosts = `-- name: GetMyPosts :many
SELECT id, post_type, title, description, phone_call, phone_text, featured_photo, creator, created_at, published_at, listed, moderator_delisted, anonymous_email, count_times_posted, count_email_responses, count_nonowner_views FROM posts_base
WHERE
    creator = ?
    AND post_type = ?
    AND moderator_delisted = false
`

type GetMyPostsParams struct {
	Creator  int                `db:"creator"`
	PostType constants.PostType `db:"post_type"`
}

func (q *Queries) GetMyPosts(ctx context.Context, arg GetMyPostsParams) ([]PostsBase, error) {
	rows, err := q.db.QueryContext(ctx, getMyPosts, arg.Creator, arg.PostType)
	if err != nil {
		return nil, err
	}
	defer rows.Close()
	var items []PostsBase
	for rows.Next() {
		var i PostsBase
		if err := rows.Scan(
			&i.ID,
			&i.PostType,
			&i.Title,
			&i.Description,
			&i.PhoneCall,
			&i.PhoneText,
			&i.FeaturedPhoto,
			&i.Creator,
			&i.CreatedAt,
			&i.PublishedAt,
			&i.Listed,
			&i.ModeratorDelisted,
			&i.AnonymousEmail,
			&i.CountTimesPosted,
			&i.CountEmailResponses,
			&i.CountNonownerViews,
		); err != nil {
			return nil, err
		}
		items = append(items, i)
	}
	if err := rows.Close(); err != nil {
		return nil, err
	}
	if err := rows.Err(); err != nil {
		return nil, err
	}
	return items, nil
}

const getPost = `-- name: GetPost :one
SELECT pb.id, pb.post_type, pb.title, pb.description, pb.phone_call, pb.phone_text, pb.featured_photo, pb.creator, pb.created_at, pb.published_at, pb.listed, pb.moderator_delisted, pb.anonymous_email, pb.count_times_posted, pb.count_email_responses, pb.count_nonowner_views, COUNT(p.post_id) as count_photos
FROM posts_base pb
LEFT JOIN photos p on p.post_id = pb.id
WHERE
    pb.id = ?
    AND pb.moderator_delisted = false
GROUP BY (p.post_id)
LIMIT 1
`

type GetPostRow struct {
	PostsBase   PostsBase `db:"posts_base"`
	CountPhotos int       `db:"count_photos"`
}

func (q *Queries) GetPost(ctx context.Context, id int) (GetPostRow, error) {
	row := q.db.QueryRowContext(ctx, getPost, id)
	var i GetPostRow
	err := row.Scan(
		&i.PostsBase.ID,
		&i.PostsBase.PostType,
		&i.PostsBase.Title,
		&i.PostsBase.Description,
		&i.PostsBase.PhoneCall,
		&i.PostsBase.PhoneText,
		&i.PostsBase.FeaturedPhoto,
		&i.PostsBase.Creator,
		&i.PostsBase.CreatedAt,
		&i.PostsBase.PublishedAt,
		&i.PostsBase.Listed,
		&i.PostsBase.ModeratorDelisted,
		&i.PostsBase.AnonymousEmail,
		&i.PostsBase.CountTimesPosted,
		&i.PostsBase.CountEmailResponses,
		&i.PostsBase.CountNonownerViews,
		&i.CountPhotos,
	)
	return i, err
}

const getPostForAnonymousEmail = `-- name: GetPostForAnonymousEmail :one
SELECT posts_base.id, posts_base.title, posts_base.post_type, users.email
FROM posts_base
JOIN users on users.id = posts_base.creator
WHERE posts_base.anonymous_email = ?
    AND listed = true
    AND moderator_delisted = false
    AND datetime(published_at) > datetime('now', ?)
`

type GetPostForAnonymousEmailParams struct {
	AnonymousEmail sql.NullString `db:"anonymous_email"`
	Datetime       interface{}    `db:"datetime"`
}

type GetPostForAnonymousEmailRow struct {
	ID       int                `db:"id"`
	Title    string             `db:"title"`
	PostType constants.PostType `db:"post_type"`
	Email    string             `db:"email"`
}

// GetPostForAnonymousEmail returns the post with a matching anonymous email field.
// Only active and published posts are returned.
func (q *Queries) GetPostForAnonymousEmail(ctx context.Context, arg GetPostForAnonymousEmailParams) (GetPostForAnonymousEmailRow, error) {
	row := q.db.QueryRowContext(ctx, getPostForAnonymousEmail, arg.AnonymousEmail, arg.Datetime)
	var i GetPostForAnonymousEmailRow
	err := row.Scan(
		&i.ID,
		&i.Title,
		&i.PostType,
		&i.Email,
	)
	return i, err
}

const getPostIDs = `-- name: GetPostIDs :many
SELECT id FROM posts_base
`

func (q *Queries) GetPostIDs(ctx context.Context) ([]int, error) {
	rows, err := q.db.QueryContext(ctx, getPostIDs)
	if err != nil {
		return nil, err
	}
	defer rows.Close()
	var items []int
	for rows.Next() {
		var id int
		if err := rows.Scan(&id); err != nil {
			return nil, err
		}
		items = append(items, id)
	}
	if err := rows.Close(); err != nil {
		return nil, err
	}
	if err := rows.Err(); err != nil {
		return nil, err
	}
	return items, nil
}

const getPublishedPosts = `-- name: GetPublishedPosts :many
SELECT pb.id, pb.title, u.email, pb.creator, pb.post_type, pb.published_at, coalesce(c.classified_type, '') as classified_type, coalesce(h.housing_type, '') as housing_type
FROM posts_base pb
JOIN users u ON u.id = pb.creator
LEFT JOIN classifieds c ON c.id = pb.id
LEFT JOIN housing h on h.id = pb.id
WHERE
    moderator_delisted = false
    AND listed = true
    AND published_at is not null
`

type GetPublishedPostsRow struct {
	ID             int                      `db:"id"`
	Title          string                   `db:"title"`
	Email          string                   `db:"email"`
	Creator        int                      `db:"creator"`
	PostType       constants.PostType       `db:"post_type"`
	PublishedAt    sql.NullTime             `db:"published_at"`
	ClassifiedType constants.ClassifiedType `db:"classified_type"`
	HousingType    constants.HousingType    `db:"housing_type"`
}

// GetPublishedPosts returns posts which are published, listed, and have not been delisted.
// classified_type is only valid when post_type == classified.
func (q *Queries) GetPublishedPosts(ctx context.Context) ([]GetPublishedPostsRow, error) {
	rows, err := q.db.QueryContext(ctx, getPublishedPosts)
	if err != nil {
		return nil, err
	}
	defer rows.Close()
	var items []GetPublishedPostsRow
	for rows.Next() {
		var i GetPublishedPostsRow
		if err := rows.Scan(
			&i.ID,
			&i.Title,
			&i.Email,
			&i.Creator,
			&i.PostType,
			&i.PublishedAt,
			&i.ClassifiedType,
			&i.HousingType,
		); err != nil {
			return nil, err
		}
		items = append(items, i)
	}
	if err := rows.Close(); err != nil {
		return nil, err
	}
	if err := rows.Err(); err != nil {
		return nil, err
	}
	return items, nil
}

const incrementEmailResponseCount = `-- name: IncrementEmailResponseCount :exec
UPDATE posts_base
SET
    count_email_responses = count_email_responses + 1
WHERE id = ?
`

// IncrementEmailResponseCount increases the counter for the number of email responses
// received for a post.
func (q *Queries) IncrementEmailResponseCount(ctx context.Context, id int) error {
	_, err := q.db.ExecContext(ctx, incrementEmailResponseCount, id)
	return err
}

const incrementNonOwnerViews = `-- name: IncrementNonOwnerViews :exec
UPDATE posts_base
SET
    count_nonowner_views = count_nonowner_views + 1
WHERE id = ?
`

// IncrementNonOwnerViews increases the counter for the number of times a post
// has been viewed by someone other than the person that created the post.
func (q *Queries) IncrementNonOwnerViews(ctx context.Context, id int) error {
	_, err := q.db.ExecContext(ctx, incrementNonOwnerViews, id)
	return err
}

const incrementTimesPosted = `-- name: IncrementTimesPosted :exec
UPDATE posts_base
SET
    count_times_posted = count_times_posted + 1
WHERE id = ?
`

// IncrementTimesPosted increases the counter for the number of times a post
// has been posted.
func (q *Queries) IncrementTimesPosted(ctx context.Context, id int) error {
	_, err := q.db.ExecContext(ctx, incrementTimesPosted, id)
	return err
}

const relistPost = `-- name: RelistPost :exec
UPDATE posts_base
SET
    published_at = datetime('now'),
    listed = true
WHERE id = ?
`

// RelistPost updates the published_at column on the post to the current time so
// that it no longer is expired and sets listed to true.
func (q *Queries) RelistPost(ctx context.Context, id int) error {
	_, err := q.db.ExecContext(ctx, relistPost, id)
	return err
}

const setPostFeaturedPhotoID = `-- name: SetPostFeaturedPhotoID :exec
UPDATE posts_base
SET featured_photo = ?
WHERE id = ?
`

type SetPostFeaturedPhotoIDParams struct {
	FeaturedPhoto conv.NullInt `db:"featured_photo"`
	ID            int          `db:"id"`
}

// SetPostFeaturedPhotoID updates the featured photo on the post.
// This should only be used on classified and for-rent posts.
func (q *Queries) SetPostFeaturedPhotoID(ctx context.Context, arg SetPostFeaturedPhotoIDParams) error {
	_, err := q.db.ExecContext(ctx, setPostFeaturedPhotoID, arg.FeaturedPhoto, arg.ID)
	return err
}

const setPostModeratorDelisted = `-- name: SetPostModeratorDelisted :exec
UPDATE posts_base
SET moderator_delisted = true
WHERE id = ?
`

// SetPostModeratorDelisted will flag this post as deleted by a moderator,
// preventing it from being seeing by any users.
func (q *Queries) SetPostModeratorDelisted(ctx context.Context, id int) error {
	_, err := q.db.ExecContext(ctx, setPostModeratorDelisted, id)
	return err
}

const setPostPublishedAt = `-- name: SetPostPublishedAt :exec
UPDATE posts_base
SET published_at = ?
WHERE id = ?
`

type SetPostPublishedAtParams struct {
	PublishedAt sql.NullTime `db:"published_at"`
	ID          int          `db:"id"`
}

// SetPostPublishedAt can be used to set the published_at column explicitly.
// This is most useful for unit tests to expire a post.
func (q *Queries) SetPostPublishedAt(ctx context.Context, arg SetPostPublishedAtParams) error {
	_, err := q.db.ExecContext(ctx, setPostPublishedAt, arg.PublishedAt, arg.ID)
	return err
}

const setPostPublishedNow = `-- name: SetPostPublishedNow :exec
UPDATE posts_base
SET published_at = datetime('now')
WHERE id = ?
`

func (q *Queries) SetPostPublishedNow(ctx context.Context, id int) error {
	_, err := q.db.ExecContext(ctx, setPostPublishedNow, id)
	return err
}

const setPostUnlisted = `-- name: SetPostUnlisted :exec
UPDATE posts_base
SET listed = false
WHERE id = ?
`

// SetPostUnlisted will set that value on the post so it no longer shows up in
// the general browser queries.
func (q *Queries) SetPostUnlisted(ctx context.Context, id int) error {
	_, err := q.db.ExecContext(ctx, setPostUnlisted, id)
	return err
}

const updatePost = `-- name: UpdatePost :exec
UPDATE posts_base
SET
    title = ?,
    description = ?
WHERE id = ?
`

type UpdatePostParams struct {
	Title       string `db:"title"`
	Description string `db:"description"`
	ID          int    `db:"id"`
}

func (q *Queries) UpdatePost(ctx context.Context, arg UpdatePostParams) error {
	_, err := q.db.ExecContext(ctx, updatePost, arg.Title, arg.Description, arg.ID)
	return err
}
