package models

import (
	"context"
	"database/sql"
	"fmt"
	"math/rand"
	"time"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
)

type UserClient struct {
	dbConfig
}

// CreateUser creates a random user entity
func (c *UserClient) CreateMockUser() (sqlc.User, error) {
	seed := fmt.Sprintf("%d-%d", time.Now().UnixMilli(), rand.Intn(1000000))

	userID, err := c.parent.C.CreateUser(context.Background(), sqlc.CreateUserParams{
		Name:  sql.NullString{Valid: true, String: fmt.Sprintf("Test User %s", seed)},
		Email: fmt.Sprintf("testuser-%s@localhost.localhost", seed),
	})
	if err != nil {
		return sqlc.User{}, err
	}

	return c.parent.C.GetUserByID(context.Background(), int(userID))
}

// SetAdmin makes the provided user a member of the "admin" group.
func (c *UserClient) SetAdmin(ctx context.Context, userID int) error {
	// TODO: Should admin be hard-coded?
	adminGroupID, err := c.parent.Group.CreateIfNotExists(ctx, "admin")
	if err != nil {
		return err
	}

	return c.parent.C.AddUserToGroup(ctx, sqlc.AddUserToGroupParams{
		UserID:  userID,
		GroupID: adminGroupID,
	})
}
