package api

import (
	"net/http"

	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
)

type deliverability struct {
	controller.Controller
}

// LoginGet returns any deliverability reports for the requested email address
// from the query parameter.
func (c *deliverability) LoginGet(ctx echo.Context) error {
	email := ctx.QueryParam("email")
	if email == "" {
		return echo.ErrBadRequest
	}

	status, ok := c.Container.Cache.Get().
		Group(constants.CachedDeliverabilityGroup).
		Key(email).
		Fetch()
	if !ok {
		status = "unknown"
	} else {
		c.Container.Cache.Flush().
			Group(constants.CachedDeliverabilityGroup).
			Key(email).
			Execute()
	}

	return ctx.JSON(
		http.StatusOK,
		map[string]string{"status": status.(string)})
}
