package classified

import (
	"database/sql"
	"fmt"
	"image/color"
	"strconv"

	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/conv"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/postbase"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views"
)

type Kiosk struct {
	controller.Controller
}

func (c *Kiosk) GetClassifieds(ctx echo.Context) error {
	htmxReq := ctx.Request().Header.Get("HX-Request")
	currentID := ctx.QueryParam("current_id")
	if currentID == "" {
		currentID = "-1"
	}

	currentIDParsed, err := strconv.Atoi(currentID)
	if err != nil {
		return c.Fail(err, "unable to retrieve post")
	}

	// TODO: Handle case where there are no posts, so the result is empty.
	post, err := c.Container.DB.C.GetNextActivePost(ctx.Request().Context(), currentIDParsed)
	if err == sql.ErrNoRows {
		// Probably wrapped around. Request again with -1 id.
		post, err = c.Container.DB.C.GetNextActivePost(ctx.Request().Context(), -1)
	}
	if err != nil {
		return c.Fail(err, "unable to retrieve post")
	}

	photos, err := c.Container.DB.C.GetPhotos(ctx.Request().Context(), post.PostsBase.ID)
	if err != nil {
		return c.Fail(err, "unable to retrieve post")
	}

	p := postbase.PostBase{
		PostsBase: post.PostsBase,
	}

	viewHref := c.Container.Config.App.PublicURL + p.GetViewHref(ctx.Echo()) + "?qr=t"
	qrCodeSVG := conv.SVGQRCode(viewHref, color.White, color.Black)

	page := controller.NewPage(ctx, c.Container)
	page.Title = fmt.Sprintf("%s Classifieds", c.Container.Config.App.Name)
	page.Metatags.Description = fmt.Sprintf("Items for sale on %s Classifieds.", c.Container.Config.App.Name)
	page.Metatags.Keywords = keywords

	v := views.KioskClassified{
		PostBase:   p,
		Classified: post.Classified,
		Photos:     photos,
		BodyOnly:   htmxReq == "true",
		QRCode:     qrCodeSVG,
		Page:       page,
	}

	// Do not allow the browser to cache these pages since it will prevent it from
	// picking up new posts in the order.
	ctx.Response().Header().Add("Cache-Control", "no-cache")

	return c.RenderHTML(ctx, page, &v)
}
