package classified

import (
	"fmt"

	"github.com/labstack/echo/v4"
	"go.uber.org/zap"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/context"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/classified"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/msg"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
)

type one struct {
	controller.Controller
}

// Get returns the details page for a single classifieds post.
func (c *one) Get(ctx echo.Context) error {
	// Middleware protects against posts not existing and being expired.
	p, ok := ctx.Get(context.PostKey).(*classified.Post)
	if !ok {
		msg.Warning(ctx, "The requested post could not be found.")
		return c.RedirectToPostBrowser(ctx)
	}

	photos, err := c.Container.DB.C.GetPhotos(ctx.Request().Context(), p.ID)
	if err != nil {
		c.Container.Logger.Warn("unable to load photos for post",
			zap.Error(err))
	}

	page := controller.NewPage(ctx, c.Container)
	page.Title = "Classifieds"
	page.Metatags.Description = fmt.Sprintf("Item on %s Classifieds.", c.Container.Config.App.Name)
	page.Metatags.Keywords = keywords

	// If the viewer is a moderator, retrieve the post creator user info to display in the moderator toolbar.
	var info sqlc.GetUserInfoRow
	if page.Enforce("post", "moderate") {
		var err error
		info, err = c.Container.DB.C.GetUserInfo(ctx.Request().Context(), p.Creator)
		if err != nil {
			msg.Warning(ctx, "Unable to retrieve post creator info")
			context.GetLogger(ctx).Warn("Unable to retrieve post creator info", zap.Error(err))
		}
	}

	if !page.IsAuth || page.AuthUser.ID != p.Creator {
		err := c.Container.DB.C.IncrementNonOwnerViews(ctx.Request().Context(), p.ID)
		if err != nil {
			c.Container.Logger.Warn("unable to increment post view count",
				zap.Error(err))
		}
	}

	v := views.PostDetails{
		PostBase:         *p.PostBase,
		Price:            p.Price,
		Photos:           photos,
		CreatorInfo:      &info,
		MarkdownRenderer: c.Container.MarkdownRenderer,
		Headers:          components.ClassifiedHeader(page, p.ClassifiedType),
		Page:             page,
	}
	return c.RenderHTML(ctx, page, &v)
}
