package classified

import (
	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/middleware"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/services"
)

var keywords = []string{"Classifieds"}

func BuildRoutes(c *services.Container, g *echo.Group, ctr controller.Controller) {
	sg := g.Group("/classified")

	authMW := middleware.RequireAuthentication()
	digestMW := middleware.WithDigestMembership(c.DB)

	// withAuth ensures that the requester has signed in.
	withAuth := sg.Group("", authMW)

	// withPost ensures that the post exists and is not expired (unless the
	// requester is the creator).
	withPost := sg.Group("", middleware.WithPost(c.DB))

	// withPostPerms ensures that the post exists and the requester is the creator.
	withPostPerms := withPost.Group("",
		authMW,
		middleware.WithPostFullPermission(c.DB, c.Enforcer))

	kiosk := Kiosk{Controller: ctr}
	sg.GET("/kiosk", kiosk.GetClassifieds).Name = pages.Names.Classified.Kiosk

	forSale := forSale{Controller: ctr}
	sg.GET("/for-sale", forSale.Get, digestMW).Name = pages.Names.Classified.ForSale.Browse
	sg.GET("/for-sale.rss", forSale.Get).Name = pages.Names.Classified.ForSale.RSS

	forFree := forFree{Controller: ctr}
	sg.GET("/for-free", forFree.Get, digestMW).Name = pages.Names.Classified.ForFree.Browse
	sg.GET("/for-free.rss", forFree.Get).Name = pages.Names.Classified.ForFree.RSS

	wtBuy := wtBuy{Controller: ctr}
	sg.GET("/want-to-buy", wtBuy.Get, digestMW).Name = pages.Names.Classified.WantToBuy.Browse
	sg.GET("/want-to-buy.rss", wtBuy.Get).Name = pages.Names.Classified.WantToBuy.RSS

	wtBorrow := wtBorrow{Controller: ctr}
	sg.GET("/want-to-borrow", wtBorrow.Get, digestMW).Name = pages.Names.Classified.WantToBorrow.Browse
	sg.GET("/want-to-borrow.rss", wtBorrow.Get).Name = pages.Names.Classified.WantToBorrow.RSS

	one := one{Controller: ctr}
	withPost.GET("/:id", one.Get).Name = pages.Names.Classified.One

	newPost := create{Controller: ctr}
	withAuth.GET("/new", newPost.Get).Name = pages.Names.Classified.New
	withAuth.POST("/new", newPost.Post)

	editPost := edit{Controller: ctr}
	withPostPerms.GET("/:id/edit", editPost.Get).Name = pages.Names.Classified.Edit
	withPostPerms.POST("/:id/edit", editPost.Post)
}
