package routes

import (
	"fmt"
	"net/http"

	"github.com/labstack/echo/v4"
	"go.uber.org/zap"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/context"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views"
)

type errorHandler struct {
	controller.Controller
}

func (e *errorHandler) Get(err error, ctx echo.Context) {
	if ctx.Response().Committed || context.IsCanceledError(err) {
		return
	}

	code := http.StatusInternalServerError
	if he, ok := err.(*echo.HTTPError); ok {
		code = he.Code
	}

	if logger, ok := ctx.Get(context.LoggerKey).(*zap.Logger); ok {
		logger.Info("displaying error page due to error in handler", zap.Error(err))
	} else {
		fmt.Printf("error and could not find logger: %s\n", err)
	}

	page := controller.NewPage(ctx, e.Container)
	page.Title = http.StatusText(code)
	page.StatusCode = code

	v := views.Error{
		Page: page,
	}
	if err = e.RenderHTML(ctx, page, &v); err != nil {
		fmt.Println("failed to render error page for error: " + err.Error())
	}
}
