package event

import (
	"fmt"
	"time"

	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/conv"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views"
)

type browser struct {
	controller.Controller
}

func (c *browser) Get(ctx echo.Context) error {
	selectedDate, err := time.Parse(time.DateOnly, ctx.QueryParam("selectedDate"))
	if err != nil {
		selectedDate = time.Now()
	}

	firstDayOfSelectedWeek := conv.FirstDayOfWeek(selectedDate)
	lastDayOfSelectedWeek := conv.LastDayOfWeek(selectedDate)

	upcomingEvents, err := c.Container.DB.C.GetUpcomingEventsInRange(ctx.Request().Context(),
		sqlc.GetUpcomingEventsInRangeParams{
			Date:   firstDayOfSelectedWeek.Format(time.DateOnly),
			Date_2: lastDayOfSelectedWeek.Format(time.DateOnly),
		})
	if err != nil {
		return c.Fail(err, "unable to retrieve upcoming events")
	}

	// TODO: Add MyEvents section

	page := controller.NewPage(ctx, c.Container)
	page.Title = "Events"
	page.Metatags.Description = c.Container.Config.App.Name + " Events Calendar"
	page.Metatags.Keywords = keywords
	page.Metatags.HasFeed = true

	v := views.EventBrowse{
		UpcomingEvents:         upcomingEvents,
		SelectedDate:           selectedDate,
		FirstDayOfSelectedWeek: firstDayOfSelectedWeek,
		LastDayOfSelectedWeek:  lastDayOfSelectedWeek,
		Page:                   page,
	}
	return c.RenderHTML(ctx, page, &v)
}

// GetRSS returns events by most recently created instead of in chronological
// order.
func (c *browser) GetRSS(ctx echo.Context) error {
	events, err := c.Container.DB.C.GetRecentlyCreatedEvents(ctx.Request().Context())
	if err != nil {
		return c.Fail(err, "unable to retrieve recently created events")
	}

	page := controller.NewPage(ctx, c.Container)
	page.Title = "Events"
	page.Metatags.Description = fmt.Sprintf("Recently created events on %s.", c.Container.Config.App.Name)
	page.Metatags.HasFeed = true

	items := make([]views.RSSItem, 0, len(events))
	for _, event := range events {
		items = append(items, views.RSSItem{
			Title:         event.Title,
			Description:   event.Description,
			Href:          c.Container.Config.App.PublicURL + ctx.Echo().Reverse(pages.Names.Event.One, event.ID),
			PublishedDate: event.CreatedAt,
		})
	}

	v := views.RSS{
		HomeHref: c.Container.Config.App.PublicURL +
			ctx.Echo().Reverse(pages.Names.Event.Browse),
		SupportEmail: c.Container.Config.Mail.SupportAddress,
		Items:        items,
		Page:         page,
		Sanitizer:    c.Container.Sanitizer,
	}
	return c.RenderXML(ctx, page, &v)
}
