package event

import (
	"fmt"
	"strconv"

	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/context"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/msg"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views"
)

type update struct {
	controller.Controller
}

func (c *update) DeleteGet(ctx echo.Context) error {
	page := controller.NewPage(ctx, c.Container)
	page.Metatags.Keywords = keywords
	page.Title = "Delete Event"
	page.Metatags.Description = fmt.Sprintf("Delete event on %s.", c.Container.Config.App.Name)

	v := views.ActionConfirmation{
		Heading:    "Delete event",
		Prompt:     "Are you sure you want to delete this event? This action cannot be undone.",
		ButtonText: "Delete",

		Page: page,
	}
	return c.RenderHTML(ctx, page, &v)
}

func (c *update) DeletePost(ctx echo.Context) error {
	eventID, err := strconv.Atoi(ctx.Param("id"))
	if err != nil {
		msg.Warning(ctx, "The requested event could not be found.")
		return c.Redirect(ctx, pages.Names.Event.Browse)
	}

	var form views.ActionConfirmationForm
	ctx.Set(context.FormKey, &form)

	if err := ctx.Bind(&form); err != nil {
		return c.Fail(err, "unable to bind form")
	}

	if err := form.Submission.Process(ctx, form); err != nil {
		return c.Fail(err, "unable to process form submission")
	}

	if form.Submission.HasErrors() {
		return c.DeleteGet(ctx)
	}

	err = c.Container.DB.C.DeleteEvent(ctx.Request().Context(), eventID)
	if err != nil {
		return c.Fail(err, "unable to delete the post")
	}

	return c.Redirect(ctx, pages.Names.Event.Browse)
}
