package event

import (
	"fmt"

	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/context"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/msg"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views"
)

func (c *update) PhotoGet(ctx echo.Context) error {
	e, ok := ctx.Get(context.EventKey).(sqlc.Event)
	if !ok {
		msg.Warning(ctx, "The requested event could not be found.")
		return c.Redirect(ctx, pages.Names.Event.Browse)
	}

	token, err := c.Container.Auth.GenerateJWTAuthToken(map[string]interface{}{
		"event_id": e.ID,
	}, &c.Container.Config.App.EmailVerificationTokenExpiration)
	if err != nil {
		return c.Fail(err, "unable to generate jwt token")
	}

	page := controller.NewPage(ctx, c.Container)
	page.Title = "Edit Event Photo"
	page.Metatags.Description = fmt.Sprintf("Edit event photo on %s.", c.Container.Config.App.Name)
	page.Metatags.Keywords = keywords

	v := views.EventPhotoUpload{
		PublishAction: ctx.Echo().Reverse(pages.Names.Event.One, e.ID),
		EventID:       e.ID,
		Token:         token,

		Page: page,
	}
	return c.RenderHTML(ctx, page, &v)
}
