package event

import (
	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/middleware"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/services"
)

var keywords = []string{"Events", "Events Calendar", "Community Events"}

func BuildRoutes(c *services.Container, g *echo.Group, ctr controller.Controller) {
	// withAuth ensures that the requester has signed in.
	withAuth := g.Group("/events", middleware.RequireAuthentication())

	// withEvent requires that the user has signed in and stores the requested event in the context.
	// Does not return events in the past.
	withEvent := withAuth.Group("", middleware.WithEvent(c.DB))

	withEventPerms := withEvent.Group("", middleware.WithEventFullPermission(c.Enforcer))

	// TODO: Add moderator delete route and handler.

	update := update{Controller: ctr}
	withEventPerms.GET("/:id/delete", update.DeleteGet).Name = pages.Names.Event.Delete
	withEventPerms.POST("/:id/delete", update.DeletePost)

	withEventPerms.GET("/:id/photos", update.PhotoGet).Name = pages.Names.Event.NewPhotos

	withEventPerms.GET("/:id/edit", update.EditGet).Name = pages.Names.Event.Edit
	withEventPerms.POST("/:id/edit", update.EditPost)

	browser := browser{Controller: ctr}
	g.GET("/events", browser.Get).Name = pages.Names.Event.Browse
	g.GET("/events.rss", browser.GetRSS).Name = pages.Names.Event.RSS

	newEvent := create{Controller: ctr}
	withAuth.GET("/new", newEvent.Get).Name = pages.Names.Event.New
	withAuth.POST("/new", newEvent.Post)

	details := details{Controller: ctr}
	withEvent.GET("/:id", details.Get).Name = pages.Names.Event.One
	withEvent.GET("/:id/add", details.AddToCalendar).Name = pages.Names.Event.AddCalendar
}
