package hooks

import "time"

type Header struct {
	Name  string `json:"Name"`
	Value string `json:"Value"`
}

// IncomingEmail is the payload expected when a webhook is called by Postmark
// for an incoming email delivery.
type IncomingEmail struct {
	From          string `json:"From"`
	MessageStream string `json:"MessageStream"`
	FromName      string `json:"FromName"`
	FromFull      struct {
		Email       string `json:"Email"`
		Name        string `json:"Name"`
		MailboxHash string `json:"MailboxHash"`
	} `json:"FromFull"`
	To     string `json:"To"`
	ToFull []struct {
		Email       string `json:"Email"`
		Name        string `json:"Name"`
		MailboxHash string `json:"MailboxHash"`
	} `json:"ToFull"`
	Cc     string `json:"Cc"`
	CcFull []struct {
		Email       string `json:"Email"`
		Name        string `json:"Name"`
		MailboxHash string `json:"MailboxHash"`
	} `json:"CcFull"`
	Bcc     string `json:"Bcc"`
	BccFull []struct {
		Email       string `json:"Email"`
		Name        string `json:"Name"`
		MailboxHash string `json:"MailboxHash"`
	} `json:"BccFull"`
	OriginalRecipient string   `json:"OriginalRecipient"`
	ReplyTo           string   `json:"ReplyTo"`
	Subject           string   `json:"Subject"`
	MessageID         string   `json:"MessageID"`
	Date              string   `json:"Date"`
	MailboxHash       string   `json:"MailboxHash"`
	TextBody          string   `json:"TextBody"`
	HTMLBody          string   `json:"HtmlBody"`
	StrippedTextReply string   `json:"StrippedTextReply"`
	Tag               string   `json:"Tag"`
	Headers           []Header `json:"Headers"`
	Attachments       []struct {
		Name          string `json:"Name"`
		Content       string `json:"Content"`
		ContentType   string `json:"ContentType"`
		ContentLength int    `json:"ContentLength"`
		ContentID     string `json:"ContentID"`
	} `json:"Attachments"`
}

// Bounce is the payload expected when a webhook is called by Postmark for a
// Bounce, Spam Complaint, or successful delivery. Not all fields in this struct
// are valid for every case, so be sure to consult the documentation.
type Bounce struct {
	RecordType    string    `json:"RecordType"`
	Type          string    `json:"Type"`
	Name          string    `json:"Name"`
	MessageID     string    `json:"MessageID"`
	ServerID      int       `json:"ServerID"`
	MessageStream string    `json:"MessageStream"`
	Description   string    `json:"Description"`
	Details       string    `json:"Details"`
	Email         string    `json:"Email"`
	Recipient     string    `json:"Recipient"`
	From          string    `json:"From"`
	BouncedAt     time.Time `json:"BouncedAt"`
	DeliveredAt   time.Time `json:"DeliveredAt"`
}
