package housing

import (
	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/middleware"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/services"
)

var keywords = []string{"Classifieds"}

func BuildRoutes(c *services.Container, g *echo.Group, ctr controller.Controller) {
	sg := g.Group("/housing")

	authMW := middleware.RequireAuthentication()
	digestMW := middleware.WithDigestMembership(c.DB)

	// withAuth ensures that the requester has signed in.
	withAuth := sg.Group("", authMW)

	// withPost ensures that the post exists and is not expired (unless the
	// requester is the creator).
	withPost := sg.Group("", middleware.WithPost(c.DB))

	// withPostPerms ensures that the post exists and the requester is the creator.
	withPostPerms := withPost.Group("",
		authMW,
		middleware.WithPostFullPermission(c.DB, c.Enforcer))

	housing := housingCtr{Controller: ctr}

	// Deprecated routes
	sg.GET("/for-rent", housing.RedirectBrowse)
	sg.GET("/request", housing.RedirectBrowse)

	sg.GET(".rss", housing.Get).Name = pages.Names.Housing.RSS
	sg.GET("", housing.Get, digestMW).Name = pages.Names.Housing.Browse
	withPost.GET("/:id", housing.GetOne).Name = pages.Names.Housing.One

	newPost := create{Controller: ctr}
	withAuth.GET("/new", newPost.Get, digestMW).Name = pages.Names.Housing.New
	withAuth.POST("/new", newPost.Post)

	edit := edit{Controller: ctr}
	withPostPerms.GET("/:id/edit", edit.Get).Name = pages.Names.Housing.Edit
	withPostPerms.POST("/:id/edit", edit.Post)
}
