package routes

import (
	"net/http"
	"testing"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"github.com/stretchr/testify/require"
)

func TestLoginTokenGet(t *testing.T) {
	// Ensure that the needed user has been instantiated
	_ = request(t).withVerifiedUser()

	token, err := c.Auth.GenerateJWTEmailAuthToken(verifiedUser.email)
	require.NoError(t, err)

	resp := request(t).
		disableFollowRedirects().
		setRoute(pages.Names.User.LoginToken, verifiedUser.id, token).
		setQueryString("?return_to=/testing").
		get()
	body := resp.body()
	t.Log(body)

	resp.requireStatusCode(http.StatusFound)
	require.Equal(t, "/testing", resp.Header.Get("Location"))

	// A request with an invalid token should redirect to the signin page
	resp = request(t).
		disableFollowRedirects().
		setRoute(pages.Names.User.LoginToken, verifiedUser.id, "badtoken").
		setQueryString("?return_to=/testing").
		get().
		requireStatusCode(http.StatusFound)
	require.Equal(t, "/user/login", resp.Header.Get("Location"))

	// A valid request with no return_to should redirect to the homepage.
	token, err = c.Auth.GenerateJWTEmailAuthToken(verifiedUser.email)
	require.NoError(t, err)

	resp = request(t).
		disableFollowRedirects().
		setRoute(pages.Names.User.LoginToken, verifiedUser.id, token).
		get().
		requireStatusCode(http.StatusFound)
	require.Equal(t, "/", resp.Header.Get("Location"))
}
