package postbase

import (
	"fmt"
	"strconv"

	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/context"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/msg"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views"
)

func (c *commonController) DeleteGet(ctx echo.Context) error {
	page := controller.NewPage(ctx, c.Container)
	page.Metatags.Keywords = keywords
	page.Title = "Delete Post"
	page.Metatags.Description = fmt.Sprintf("Delete a post on %s.", c.Container.Config.App.Name)

	v := views.ActionConfirmation{
		Heading:     "Delete post",
		Prompt:      "Are you sure you want to delete this post? This action cannot be undone.",
		ButtonText:  "Delete",
		Destructive: true,

		Page: page,
	}
	return c.RenderHTML(ctx, page, &v)
}

func (c *commonController) DeletePost(ctx echo.Context) error {
	postID, err := strconv.Atoi(ctx.Param("id"))
	if err != nil {
		msg.Warning(ctx, "The requested post could not be found.")
		return c.RedirectToPostBrowser(ctx)
	}

	var form views.ActionConfirmationForm
	ctx.Set(context.FormKey, &form)

	if err := ctx.Bind(&form); err != nil {
		return c.Fail(err, "unable to bind form")
	}

	if err := form.Submission.Process(ctx, form); err != nil {
		return c.Fail(err, "unable to process form submission")
	}

	if form.Submission.HasErrors() {
		return c.DeleteGet(ctx)
	}

	err = c.Container.DB.C.DeletePost(ctx.Request().Context(), postID)
	if err != nil {
		return c.Fail(err, "unable to delete the post")
	}

	return c.RedirectToPostBrowser(ctx)
}
