package postbase

import (
	"fmt"
	"strconv"

	"github.com/labstack/echo/v4"
	"go.uber.org/zap"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/context"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/msg"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views"
)

func (c *commonController) DelistGet(ctx echo.Context) error {
	page := controller.NewPage(ctx, c.Container)
	page.Metatags.Keywords = keywords
	page.Title = "Delist Post"
	page.Metatags.Description = fmt.Sprintf("Delist post on %s.", c.Container.Config.App.Name)

	v := views.ActionConfirmation{
		Heading:     "Delist post",
		Prompt:      "Are you sure you want to mark this post as sold or unavailable?",
		ButtonText:  "Delist",
		Destructive: true,

		Page: page,
	}
	return c.RenderHTML(ctx, page, &v)
}

func (c *commonController) DelistPost(ctx echo.Context) error {
	postID, err := strconv.Atoi(ctx.Param("id"))
	if err != nil {
		msg.Warning(ctx, "The requested post could not be found.")
		return c.RedirectToPostBrowser(ctx)
	}

	var form views.ActionConfirmationForm
	ctx.Set(context.FormKey, &form)

	if err := ctx.Bind(&form); err != nil {
		return c.Fail(err, "unable to bind form")
	}

	if err := form.Submission.Process(ctx, form); err != nil {
		return c.Fail(err, "unable to process form submission")
	}

	if form.Submission.HasErrors() {
		return c.DelistGet(ctx)
	}

	err = c.Container.DB.C.SetPostUnlisted(ctx.Request().Context(), postID)
	if err != nil {
		return c.Fail(err, "unable to delist the post")
	}

	err = c.Container.DB.C.IncrementStat(ctx.Request().Context(), constants.PostDelisted)
	if err != nil {
		context.GetLogger(ctx).Error("unable to increment stat for delisted post",
			zap.Error(err))
	}

	return c.RedirectToPostBrowser(ctx)
}
