package postbase

import (
	"errors"
	"fmt"

	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/context"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views"
)

type myPostsController struct {
	controller.Controller
}

func (c *myPostsController) Get(ctx echo.Context) error {
	page := controller.NewPage(ctx, c.Container)
	page.Title = "My Posts"
	page.Metatags.Description = fmt.Sprintf("My posts on %s.", c.Container.Config.App.Name)
	page.Metatags.Keywords = keywords

	v := views.AccountMyPosts{
		Page: page,
	}

	err := c.loadPosts(ctx, &v)
	if err != nil {
		if he, ok := err.(*echo.HTTPError); ok {
			return he
		}
		return c.Fail(err, "unable to retrieve requested post")
	}
	return c.RenderHTML(ctx, page, &v)
}

func (c *myPostsController) loadPosts(ctx echo.Context, v *views.AccountMyPosts) error {
	user, ok := ctx.Get(context.AuthenticatedUserKey).(sqlc.User)
	if !ok {
		return errors.New("missing authenticated user, unable to load posts")
	}

	realCtx := ctx.Request().Context()
	var err error

	v.ForSale, err = c.Container.DB.Classified.MyPosts(realCtx, constants.ForSale, user.ID)
	if err != nil {
		return err
	}
	v.ForFree, err = c.Container.DB.Classified.MyPosts(realCtx, constants.ForFree, user.ID)
	if err != nil {
		return err
	}
	v.WantToBuy, err = c.Container.DB.Classified.MyPosts(realCtx, constants.WantToBuy, user.ID)
	if err != nil {
		return err
	}
	v.WantToBorrow, err = c.Container.DB.Classified.MyPosts(realCtx, constants.WantToBorrow, user.ID)
	if err != nil {
		return err
	}

	v.ForRent, err = c.Container.DB.Housing.MyPosts(ctx.Request().Context(), constants.ForRent, user.ID)
	if err != nil {
		return err
	}
	v.Request, err = c.Container.DB.Housing.MyPosts(ctx.Request().Context(), constants.Request, user.ID)
	if err != nil {
		return err
	}

	return nil
}
