package postbase

import (
	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/middleware"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/services"
)

const (
	ID = "id"
)

type commonController struct {
	controller.Controller
}

func BuildRoutes(c *services.Container, g *echo.Group, ctr controller.Controller) {
	sg := g.Group("/post")

	withAuth := sg.Group("",
		middleware.RequireAuthentication())

	// withPostPerms ensures that the post exists and the requester is the creator.
	withPostPerms := withAuth.Group("",
		middleware.WithPost(c.DB),
		middleware.WithPostFullPermission(c.DB, c.Enforcer))

	withPostModerator := sg.Group("",
		middleware.RequirePostModerator(c.DB, c.Enforcer),
		middleware.WithPost(c.DB))

	myPostsController := myPostsController{Controller: ctr}
	withAuth.GET("/my", myPostsController.Get).Name = pages.Names.Posts.MyPosts

	commonCtr := commonController{Controller: ctr}

	withPostPerms.GET("/:id/photos", commonCtr.PhotosGet).Name = pages.Names.Posts.NewPhotos
	withPostPerms.POST("/:id/photos/:photo-id/delete", commonCtr.PhotoDelete).Name = pages.Names.Posts.DeletePhoto

	withPostPerms.POST("/:id/publish", commonCtr.PublishPost).Name = pages.Names.Posts.Publish

	withPostPerms.GET("/:id/delist", commonCtr.DelistGet).Name = pages.Names.Posts.Delist
	withPostPerms.POST("/:id/delist", commonCtr.DelistPost)

	withPostPerms.GET("/:id/relist", commonCtr.RelistGet).Name = pages.Names.Posts.Relist
	withPostPerms.POST("/:id/relist", commonCtr.RelistPost)

	withPostPerms.GET("/:id/delete", commonCtr.DeleteGet).Name = pages.Names.Posts.Delete
	withPostPerms.POST("/:id/delete", commonCtr.DeletePost)

	withPostModerator.GET("/:id/moderate/delete", commonCtr.ModeratorDeleteGet).Name = pages.Names.Posts.ModeratorDelete
	withPostModerator.POST("/:id/moderate/delete", commonCtr.ModeratorDeletePost)
}
