package routes

import (
	"database/sql"

	"github.com/labstack/echo/v4"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/msg"
)

type unsubscribe struct {
	controller.Controller
}

// Get handles unsubscribing a user from various resources based on the contents
// of the JWT
func (c *unsubscribe) Get(ctx echo.Context) error {
	token := ctx.QueryParam("token")
	if token == "" {
		msg.Warning(ctx, "Unable to find token")
		return c.Redirect(ctx, pages.Names.Home)
	}

	claims, err := c.Container.Auth.ValidateJWTAuthTokenMap(token)
	if err != nil {
		msg.Warning(ctx, "Invalid unsubscribe token")
		return c.Redirect(ctx, pages.Names.Home)
	}

	userID, ok := claims["user_id"].(float64)
	if !ok {
		msg.Warning(ctx, "Invalid unsubscribe token")
		return c.Redirect(ctx, pages.Names.Home)
	}

	action := claims["action"].(string)
	switch action {
	case "digest":
		err := c.unsubscribeDigestEmails(ctx, int(userID))
		if err != nil {
			msg.Danger(ctx, "Unable to unsubscribe from digest emails")
			return c.Redirect(ctx, pages.Names.Home)
		}
	default:
		msg.Warning(ctx, "Invalid unsubscribe token")
		return c.Redirect(ctx, pages.Names.Home)
	}

	msg.Success(ctx, "Successfully unsubscribed")
	return c.Redirect(ctx, pages.Names.Home)
}

func (c *unsubscribe) unsubscribeDigestEmails(ctx echo.Context, userID int) error {
	return c.Container.DB.C.SetUserSetting(ctx.Request().Context(),
		sqlc.SetUserSettingParams{
			UserID: userID,
			Key:    string(constants.EmailDigestSubscription),
			Value: sql.NullString{
				Valid:  true,
				String: constants.EmailDigestSubscriptionUnsubscribed,
			},
		})
}
