package services

import (
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestCacheClient(t *testing.T) {
	type cacheTest struct {
		Value string
	}
	// Cache some data
	data := cacheTest{Value: "abcdef"}
	group := "testgroup"
	key := "testkey"
	ok := c.Cache.
		Set().
		Group(group).
		Key(key).
		Data(data).
		Save()
	require.True(t, ok)

	c.Cache.Wait()

	// Get the data
	fromCache, ok := c.Cache.
		Get().
		Group(group).
		Key(key).
		Fetch()
	require.True(t, ok)
	cast, ok := fromCache.(cacheTest)
	require.True(t, ok)
	assert.Equal(t, data, cast)

	// The same key with the wrong group should fail
	_, ok = c.Cache.
		Get().
		Key(key).
		Fetch()
	assert.False(t, ok)

	// Flush the data
	c.Cache.
		Flush().
		Group(group).
		Key(key).
		Execute()

	// The data should be gone
	assertFlushed := func() {
		// The data should be gone
		_, ok = c.Cache.
			Get().
			Group(group).
			Key(key).
			Fetch()
		assert.False(t, ok)
	}
	assertFlushed()

	// Set with expiration
	ok = c.Cache.
		Set().
		Group(group).
		Key(key).
		Data(data).
		Expiration(time.Millisecond).
		Save()
	assert.True(t, ok)

	// Wait for expiration
	time.Sleep(time.Millisecond * 2)

	// The data should be gone
	assertFlushed()
}
