package types

type (
	MessageStream string
	ContentType   string
)

const (
	// NOTE: If adding any additional message streams, they must be configured
	// in pkg/postmark/postmark.go.
	AccountManagement   MessageStream = "account-management"
	AdminNotifications  MessageStream = "admin-notifications"
	TwiceWeeklyDigest   MessageStream = "twice-weekly-digest"
	EventsTransactional MessageStream = "events-transactional"
	UserMessages        MessageStream = "user-messages"

	Calendar ContentType = "text/calendar; charset=utf-8; method=REQUEST"
)

type Mail struct {
	MessageStream MessageStream
	From          string
	To            string
	ReplyTo       string
	Subject       string
	HTMLBody      string
	TextBody      string
	Attachments   []Attachment
	Headers       []Header
}

type Header struct {
	Name  string `json:"Name"`
	Value string `json:"Value"`
}

type Attachment struct {
	Name        string      `json:"Name"`
	ContentType ContentType `json:"ContentType"`
	Disposition string      `json:"Disposition"`

	// Content should be base64 encoded
	Content string `json:"Content"`
}

type Response struct {
	ErrorCode int
	Message   string
	MessageID string
	To        string
}
