package services

import (
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestSanitizer(t *testing.T) {
	testcases := []struct {
		Name     string
		Input    string
		Expected string
	}{
		{
			Name:     "Field is clean",
			Input:    "this is a test",
			Expected: "this is a test",
		},
		{
			Name:     "SQL injection",
			Input:    "password' OR 5=5",
			Expected: "password&#39; OR 5=5",
		},
		{
			Name:     "XSS attack",
			Input:    `<a onblur="alert(secret)" href="http://www.google.com">Google</a>`,
			Expected: `Google`,
		},
	}

	for _, tc := range testcases {
		t.Run(tc.Name, func(t *testing.T) {
			result := c.Sanitizer.Sanitize(tc.Input)
			assert.Equal(t, tc.Expected, result)
		})
	}
}
