package services

import (
	"os"
	"testing"

	"github.com/labstack/echo/v4"
	"go.uber.org/zap"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/tests"
)

var (
	c   *Container
	ctx echo.Context
	usr sqlc.User
)

func TestMain(m *testing.M) {
	// Create a new container
	configPath := tests.LoadConfigPath()
	c = NewContainer(configPath, zap.NewNop())

	// Create a web context
	ctx, _ = tests.NewContext(c.Web, "/")
	tests.InitSession(ctx)

	// Create a test user
	var err error
	if usr, err = c.DB.User.CreateMockUser(); err != nil {
		panic(err)
	}

	// Run tests
	exitVal := m.Run()

	// Shutdown the container
	if err = c.Shutdown(); err != nil {
		panic(err)
	}

	os.Exit(exitVal)
}
