package services

import (
	"regexp"

	"github.com/asaskevich/govalidator"
)

// phoneTester matches a phone number of the format 123-456-7890.
// The UI masks the input to this format, so this is the only format we are
// expecting.
var phoneTester = regexp.MustCompile(`^\d{3}-\d{3}-\d{4}$`)

// moneyTester matches a correctly formatted dollar amount, excluding the dollar
// sign. The decimal is only allowed if there are values after it.
var moneyTester = regexp.MustCompile(`^(?:\d{1,3},)*\d+(?:\.\d{1,2})?$`)

var bathroomCountTester = regexp.MustCompile(`^\d*(?:.(?:00?|25|50?|75))?$`)

// Validator provides validation mainly validating structs within the web context
type Validator struct{}

// NewValidator creats a new Validator
func NewValidator() *Validator {
	// Add custom validators
	govalidator.TagMap["phone"] = govalidator.Validator(func(str string) bool {
		return phoneTester.Match([]byte(str))
	})

	govalidator.TagMap["money"] = govalidator.Validator(func(str string) bool {
		return moneyTester.Match([]byte(str))
	})

	govalidator.TagMap["bathroomCount"] = govalidator.Validator(func(str string) bool {
		return bathroomCountTester.Match([]byte(str))
	})

	return &Validator{}
}

// Validate validates a struct
func (v *Validator) Validate(i interface{}) error {
	// TODO: What is the error message for the custom validators?

	ok, err := govalidator.ValidateStruct(i)
	if ok {
		return nil
	}
	return err
}
