package views

import (
	"io"
	"net/http"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"
)

type AccountUpdateForm struct {
	EmailDigests string `form:"email_digests" valid:"required"`

	Submission controller.FormSubmission
}

type Account struct {
	Form             AccountUpdateForm
	BillingPortalURL string
	Page             controller.Page
}

func (v *Account) Render(w io.Writer) error {
	content := c.WideContainer(
		c.CenteredX(
			c.FullScreenForm(
				http.MethodPost,
				v.Page.Path,
				"",
				"",
				c.FormFooter(nil, "", "Save Settings", false),
				c.FormSection(
					c.FormHeading("Account Settings", ""),
					c.CSRF(v.Page),
					c.InputText(c.FormInputParams{
						Name:     "email",
						Display:  "Email Address",
						Required: true,
						Value:    v.Page.AuthUser.Email,
						Disabled: true,
					}),
					c.InputSelect(c.FormSelectParams{
						FormInputParams: c.FormInputParams{
							Name:    "email_digests",
							Display: "Classified Email Digests",
							Value:   v.Form.EmailDigests,
						},
						Options: []c.FormSelectOption{
							{
								Display: "Twice Weekly",
								Value:   "twice-weekly-digest",
							},
							{
								Display: "Unsubscribed",
								Value:   "unsubscribed",
							},
						},
					}))),
			h.Div(
				h.Class("my-6 space-y-1"),
				c.ListHeader("Manage Donations"),
				c.TextDescription("OrcasHub uses Stripe to process and manage donations."),
				h.Div(
					h.Class("pt-4 flex justify-end space-x-3"),
					c.LazyIf(v.BillingPortalURL != "", func() g.Node {
						return c.ButtonSecondary(v.BillingPortalURL, g.Text("Manage Your Donation"))
					}),
					c.ButtonPrimary(v.Page.ToURL(pages.Names.Donate), g.Text("Donate Now")))),
			c.FullScreenForm(
				http.MethodGet,
				v.Page.ToURL(pages.Names.User.DeleteAccount),
				"",
				"",
				c.FormFooter(nil, "", "Delete Account", true),
				c.FormSection(
					c.FormHeading("Delete Account",
						"Deleting your account is immediate and permanent. All posts and content you created will be removed.",
					))),
		),
	)

	l := layouts.Primary{
		Content: content,
		Page:    v.Page,
	}
	return l.Render(w)
}
