package views

import (
	"fmt"
	"io"

	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/postbase"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
)

type AccountMyPosts struct {
	ForSale      []postbase.PostBase
	ForFree      []postbase.PostBase
	WantToBuy    []postbase.PostBase
	WantToBorrow []postbase.PostBase

	ForRent []postbase.PostBase
	Request []postbase.PostBase

	Page controller.Page
}

func (v *AccountMyPosts) Render(w io.Writer) error {
	newClassifiedHref := v.Page.ToURL(pages.Names.Classified.New)
	newForSaleHref := fmt.Sprintf("%s?src_classified_type=%s", newClassifiedHref, constants.ForSale)
	newForFreeHref := fmt.Sprintf("%s?src_classified_type=%s", newClassifiedHref, constants.ForFree)
	newLookingForHref := fmt.Sprintf("%s?src_classified_type=%s", newClassifiedHref, constants.WantToBuy)
	newWTBHref := fmt.Sprintf("%s?src_classified_type=%s", newClassifiedHref, constants.WantToBorrow)
	newHousingHref := v.Page.ToURL(pages.Names.Housing.New)

	// TODO: Include a new post link in each group
	content := c.WideContainer(
		h.Div(
			c.PrimaryHeader("Classifieds"),
			// TODO: Should these be constants which know how to render themselves?
			v.section("For Sale", newForSaleHref, v.ForSale),
			v.section("For Free", newForFreeHref, v.ForFree),
			v.section("Looking For", newLookingForHref, v.WantToBuy),
			v.section("Want to Borrow", newWTBHref, v.WantToBorrow)),
		h.Div(
			h.Class("pt-8"),
			c.PrimaryHeader("Housing"),
			v.section("Housing Available", newHousingHref, v.ForRent),
			v.section("Housing Requests", newHousingHref, v.Request),
		),
	)

	l := layouts.Primary{
		Content: content,
		Page:    v.Page,
	}
	return l.Render(w)
}

func (v *AccountMyPosts) section(name, newPostHref string, posts []postbase.PostBase) g.Node {
	table := MyPostsTable{Page: v.Page}

	return h.Div(
		h.Div(
			h.Class("pt-8"),
			c.TertiaryHeader(name)),
		table.Table(posts, newPostHref),
	)
}
