package views

import (
	"io"

	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
)

type CommunityResources struct {
	Page controller.Page
}

func (v *CommunityResources) Render(w io.Writer) error {
	content := h.Div(
		c.NarrowContainer(
			c.PrimaryHeader("Community Resources"),
			h.Div(
				h.Class("mt-8 grid grid-cols-1 sm:grid-cols-2"),
				MakeSurroundingNode("Activities", "https://groups.orcashub.org/pub/activities", c.LucideTentTree()),
				MakeSurroundingNode("Events", v.Page.ToURL(pages.Names.Event.Browse), c.LucideCalendarDays()),
				MakeSurroundingNode("People", "https://groups.orcashub.org/u?period=all&order=username&asc=true", c.LucideUsers()),
				MakeSurroundingNode("Nonprofits", "https://groups.orcashub.org/pub/non-profit-organizations", c.LucideHeartHandshake()),
				MakeSurroundingNode("Housing", v.Page.ToURL(pages.Names.Housing.Browse), c.LucideHouse()),
				MakeSurroundingNode("Public Services", "https://groups.orcashub.org/pub/public-services", c.LucideTrafficCone()),
				// MakeSurroundingNode("Things to do Inside", "https://groups.orcashub.org/pub/things-to-do-inside", c.LucideDramaMask()),
				// MakeSurroundingNode("Emergency Services", "", c.LucideAmbulance()),
				// MakeSurroundingNode("Utilities", "", c.LucideUtilityPole()),
				// MakeSurroundingNode("Businesses", "", c.LucideStore()),
				// MakeSurroundingNode("Transportation", "", c.LucideCar()),
				// MakeSurroundingNode("Home Services", "", c.LucideDrill()),
				// MakeSurroundingNode("Food", "", c.LucideApple()),
				// MakeSurroundingNode("News", "", c.LucideNewspaper()),
			),
		),
	)

	l := layouts.Primary{
		Content: content,
		Page:    v.Page,
	}
	return l.Render(w)
}

func MakeSurroundingNode(name, href string, image g.Node) g.Node {
	return h.Div(
		h.Class("sm:px-6 lg:px-4 xl:mp-8 py-4 text-gray-700 hover:text-gray-900 hover:bg-gray-50 rounded-md"),
		h.A(
			h.Href(href),
			h.Div(
				h.Class("flex items-center gap-x-4"),
				h.Div(
					h.Class("h-16 w-16"),
					image,
				),
				h.Span(
					h.Class("flex-grow"),
					g.Text(name),
				),
			),
		),
	)
}
