package components

import (
	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"
)

// ButtonPrimary is an anchor button of the primary color which
// navigates to the provided href when clicked.
func ButtonPrimary(href string, children ...g.Node) g.Node {
	return h.A(
		h.Href(href),
		h.Class("inline-flex items-center justify-center gap-x-2 rounded-md border border-transparent bg-indigo-600 px-3 py-2 font-medium leading-4 text-white shadow-sm hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-indigo-500 focus:ring-offset-2 sm:text-sm"),
		g.Group(children))
}

func ButtonSecondary(href string, children ...g.Node) g.Node {
	return h.A(
		h.Href(href),
		h.Class("inline-flex items-center justify-center gap-x-2 rounded-md border border-gray-300 bg-white px-3 py-2 font-medium leading-4 text-gray-700 shadow-sm hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-indigo-500 focus:ring-offset-2 sm:text-sm"),
		g.Group(children))
}

func ButtonDestructive(href string, children ...g.Node) g.Node {
	return h.A(
		h.Href(href),
		h.Class("inline-flex items-center justify-center gap-x-2 rounded-md border border-transparent bg-red-100 px-3 py-2 font-medium leading-4 text-red-700 shadow-sm hover:bg-red-200 focus:outline-none focus:ring-2 focus:ring-red-500 focus:ring-offset-2 sm:text-sm"),
		g.Group(children))
}

func ButtonCancelBack() g.Node {
	return h.Button(
		XClick("history.back()"),
		h.Type("button"),
		h.Class("inline-flex items-center justify-center gap-x-2 rounded-md border border-gray-300 bg-white px-3 py-2 font-medium leading-4 text-gray-700 shadow-sm hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-indigo-500 focus:ring-offset-2 sm:text-sm"),
		g.Text("Cancel"))
}

func ButtonCancel(href string) g.Node {
	return ButtonSecondary(href, g.Text("Cancel"))
}

type ButtonParams struct {
	Href    string
	Display string
}

func FloatingToolbarButton(params ButtonParams, classes string) g.Node {
	// border-gray-300 bg-white text-indigo-600 hover:bg-indigo-50 focus:border-indigo-500 focus:ring-indigo-500

	return h.Div(
		h.Class("mx-2"),
		h.A(
			h.Class("relative inline-flex items-center rounded-md border px-4 py-2 text-sm font-medium focus:z-10 focus:outline-none focus:ring-1 "+classes),
			h.Href(params.Href),
			g.Text(params.Display)))
}

func FloatingToolbarJoinedButtons(left, right ButtonParams, classes string) g.Node {
	// border-gray-300 bg-white text-indigo-600 hover:bg-indigo-50 focus:border-indigo-500 focus:ring-indigo-500

	return h.Div(
		h.Class("mx-2"),
		h.A(
			h.Class("relative inline-flex items-center rounded-l-md border px-4 py-2 text-sm font-medium focus:z-10 focus:outline-none focus:ring-1 "+classes),
			h.Href(left.Href),
			g.Text(left.Display)),
		h.A(
			h.Class("relative inline-flex items-center rounded-r-md border px-4 py-2 text-sm font-medium focus:z-10 focus:outline-none focus:ring-1 "+classes),
			h.Href(right.Href),
			g.Text(right.Display)))
}
