package components

import g "maragu.dev/gomponents"

type Lazy func() g.Node

type LazyCond struct {
	Cond   bool
	Result Lazy
}

// LazyIf evaluates ifTrue and returns it only if the condition is true.
// Otherwise it returns nil.
func LazyIf(cond bool, ifTrue Lazy) g.Node {
	if cond {
		return ifTrue()
	}
	return nil
}

// LazyIfElse evaluates and returns ifTrue if the condition is true, otherwise
// it does the same with ifFalse.
func LazyIfElse(cond bool, ifTrue, ifFalse Lazy) g.Node {
	if cond {
		return ifTrue()
	}
	return ifFalse()
}

// LazySwitch considers the conditions in order, evaluating and returning the
// first true condition. If all are false, nil is returned.
func LazySwitch(conditions ...LazyCond) g.Node {
	for _, cond := range conditions {
		if cond.Cond {
			return cond.Result()
		}
	}
	return nil
}
