package components

import (
	g "maragu.dev/gomponents"
	gc "maragu.dev/gomponents/components"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
)

func ClassifiedHeader(page controller.Page, currentPostType constants.ClassifiedType) g.Node {
	digestButtonURL := page.ToURL(pages.Names.User.JoinDigest)
	if !page.IsAuth {
		digestButtonURL = page.ToURL(pages.Names.User.Login)
	}

	return g.Group([]g.Node{
		HeaderContainer(
			g.Group([]g.Node{
				HeaderButton("For Sale", page.ToURL(pages.Names.Classified.ForSale.Browse),
					currentPostType == constants.ForSale),
				HeaderButton("For Free", page.ToURL(pages.Names.Classified.ForFree.Browse),
					currentPostType == constants.ForFree),
				HeaderButton("Looking For", page.ToURL(pages.Names.Classified.WantToBuy.Browse),
					currentPostType == constants.WantToBuy),
				HeaderButton("Want to Borrow", page.ToURL(pages.Names.Classified.WantToBorrow.Browse),
					currentPostType == constants.WantToBorrow),
				HeaderButton("Housing", page.ToURL(pages.Names.Housing.Browse), false),
				LazyIf(page.DisplayEmailDigestCTA, func() g.Node {
					return EmphasizedHeaderButton("Email Digest", digestButtonURL, false, false)
				}),
			}),
			HeaderButton("New Post",
				page.ToURL(pages.Names.Classified.New)+"?src_classified_type="+string(currentPostType),
				page.Path == page.ToURL(pages.Names.Classified.New))),
		LazyIf(page.DisplayEmailDigestCTA, func() g.Node {
			return DigestBanner(digestButtonURL)
		}),
	})
}

func HousingHeader(page controller.Page) g.Node {
	digestButtonURL := page.ToURL(pages.Names.User.JoinDigest)
	if !page.IsAuth {
		digestButtonURL = page.ToURL(pages.Names.User.Login)
	}

	return g.Group([]g.Node{
		HeaderContainer(
			g.Group([]g.Node{
				HeaderButton("For Sale", page.ToURL(pages.Names.Classified.ForSale.Browse), false),
				HeaderButton("For Free", page.ToURL(pages.Names.Classified.ForFree.Browse), false),
				HeaderButton("Looking For", page.ToURL(pages.Names.Classified.WantToBuy.Browse), false),
				HeaderButton("Want to Borrow", page.ToURL(pages.Names.Classified.WantToBorrow.Browse), false),
				HeaderButton("Housing", page.ToURL(pages.Names.Housing.Browse),
					page.Path != page.ToURL(pages.Names.Housing.New)),
				LazyIf(page.DisplayEmailDigestCTA, func() g.Node {
					return EmphasizedHeaderButton("Email Digest", digestButtonURL, false, false)
				}),
			}),
			HeaderButton("New Post",
				page.ToURL(pages.Names.Housing.New),
				page.Path == page.ToURL(pages.Names.Housing.New))),
		LazyIf(page.DisplayEmailDigestCTA, func() g.Node {
			return DigestBanner(digestButtonURL)
		}),
	})
}

func EventHeader(page controller.Page) g.Node {
	return HeaderContainer(
		nil,
		HeaderButton("New Event",
			page.ToURL(pages.Names.Event.New),
			page.Path == page.ToURL(pages.Names.Event.New)))
}

func HeaderContainer(startLinks, endLinks g.Node) g.Node {
	return h.Header(
		h.Class("bg-white shadow-sm"),
		h.Div(
			h.Class("mx-auto max-w-7xl border-b-2 border-gray-200 px-4 sm:border-none sm:px-6 lg:px-8"),
			h.Div(
				h.Class("h-15 flex items-center justify-between"),
				h.Div(
					h.Class("flex justify-start"),
					startLinks),
				h.Div(
					h.Class("flex justify-end"),
					endLinks))))
}

func HeaderButton(display, href string, current bool) g.Node {
	return h.A(
		h.Href(href),
		gc.Classes{
			"border-b-2 px-3 py-5 text-sm font-medium text-center sm:text-left":          true,
			"border-indigo-600 text-indigo-600":                                          current,
			"border-transparent text-gray-700 hover:border-gray-300 hover:text-gray-800": !current,
		},
		g.Text(display))
}

func EmphasizedHeaderButton(display, href string, current bool, displayOnMobile bool) g.Node {
	return h.A(
		h.Href(href),
		gc.Classes{
			"border-b-2 px-3 py-5 text-sm font-semibold text-center sm:text-left":        true,
			"border-indigo-600 text-indigo-600":                                          current,
			"border-transparent text-gray-700 hover:border-gray-300 hover:text-gray-800": !current,
			"hidden sm:block": !displayOnMobile,
		},
		g.Text(display))
}

type BreadcrumbPart struct {
	Display string
	Href    string
}

func Breadcrumbs(parts []BreadcrumbPart) g.Node {
	listElements := make([]g.Node, 0, (len(parts)*2)-1)
	for i, part := range parts {
		if i != 0 {
			listElements = append(listElements, IconChevronRight(h.Class("min-w-fit h-3 w-3")))
		}

		listElements = append(listElements,
			h.Li(
				h.Class("min-w-fit"),
				LazyIfElse(part.Href != "",
					func() g.Node {
						return h.A(
							h.Class("hover:text-gray-900 focus:outline-none"),
							h.Href(part.Href),
							g.Text(part.Display))
					},
					func() g.Node {
						return h.P(
							h.Class("rounded active-breadcrumb"),
							g.Text(part.Display))
					})))
	}

	return h.Div(
		h.Class("px-3 py-5"),
		h.Ol(
			h.Class("inline-flex items-center space-x-1 text-xs text-gray-400 [&_.active-breadcrumb]:text-gray-600 [&_.active-breadcrumb]:font-medium"),
			g.Group(listElements)))
}

func DigestBanner(digestButtonURL string) g.Node {
	return h.Div(
		XCloak(),
		XData(`{show: $persist(true).as('show_digest_banner').using(sessionStorage)}`),
		XShow(`show`),
		h.Class("flex items-center bg-indigo-600 px-6 py-2 sm:hidden"),
		h.P(
			h.Class("text-sm text-white"),
			h.A(
				h.Href(digestButtonURL),
				g.Text("Join the twice-weekly "),
				h.Span(
					h.Class("underline"),
					g.Text("email digest")))),
		// IconArrowRight(h.Class("h-5 w-5 inline-block")))),
		h.Div(
			h.Class("flex flex-1 justify-end text-white"),
			h.Button(
				XClick(`show = false`),
				IconMiniX(
					h.Class("h-5 w-5")),
			)))
}
