package components

import (
	"strconv"

	g "maragu.dev/gomponents"
)

// HxGet is the shorthand for HTMX get directive.
// https://htmx.org/attributes/hx-get/
func HxGet(path string) g.Node {
	return RawAttr("hx-get", path)
}

// HxPatch is the shorthand for HTMX patch directive.
// https://htmx.org/attributes/hx-patch/
func HxPatch(path string) g.Node {
	return RawAttr("hx-patch", path)
}

// HxTrigger is the shorthand for HTMX trigger directive.
// https://htmx.org/attributes/hx-trigger/
func HxTrigger(trigger string) g.Node {
	return RawAttr("hx-trigger", trigger)
}

// HxSwap is the shorthand for HTMX swap directive.
// https://htmx.org/attributes/hx-swap/
func HxSwap(target string) g.Node {
	return RawAttr("hx-swap", target)
}

// HxTarget is the shorthand for HTMX target directive.
// https://htmx.org/attributes/hx-target/
func HxTarget(selector string) g.Node {
	return RawAttr("hx-target", selector)
}

// HxSync is the shorthand for HTMX sync directive.
// https://htmx.org/attributes/hx-sync/
func HxSync(strategy string) g.Node {
	return RawAttr("hx-sync", strategy)
}

// HxPushURL is the shorthand for HTMX push url directive.
// https://htmx.org/attributes/hx-push-url/
// url param is optional
func HxPushURL(value bool, url string) g.Node {
	if url != "" {
		return RawAttr("hx-push-url", url)
	}

	return RawAttr("hx-push-url", strconv.FormatBool(value))
}
