package components

import (
	"github.com/labstack/echo/v4"
	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/postbase"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/sqlc"
)

var badges = map[constants.HousingType]g.Node{
	constants.ForRent: GreenBadge("For Rent"),
	constants.Request: BlueBadge("Request"),
}

// EmptyBrowserPlaceholder can be displayed when a browse page has no content
// currently browsable. It displays a nice message along with a button to create
// new a new item of this type.
func EmptyBrowserPlaceholder(buttonText, newPostHref string) g.Node {
	return h.Div(
		h.Class("py-32 text-center"),
		h.H3(
			h.Class("mt-2 text-sm font-medium text-gray-900"),
			g.Text("No posts")),
		TextDescription("There's nothing here yet.", "mt-1"),
		h.Div(
			h.Class("mt-6"),
			h.A(
				h.Href(newPostHref),
				h.Class("inline-flex items-center rounded-md border border-transparent bg-indigo-600 px-4 py-2 text-sm font-medium text-white shadow-sm hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-indigo-500 focus:ring-offset-2"),
				IconSolidPlus(
					h.Class("-ml-1 mr-2 h-5 w-5")),
				g.Text(buttonText))))
}

// BrowserList displays the provided items in a list. It does not handle if
// there are no items in the list.
func BrowserList(ctx echo.Context, items []postbase.PostBase) g.Node {
	return h.Div(
		h.Class("grid grid-cols-1 gap-4 sm:grid-cols-2"),
		g.Group(g.Map(items,
			func(item postbase.PostBase) g.Node {
				// This function only works for Housing posts right now.
				if item.PostType != constants.Housing {
					return nil
				}

				return h.Div(
					h.Class("relative flex items-center space-x-3 rounded-lg border border-gray-300 bg-white px-6 py-5 shadow-sm focus-within:ring-2 focus-within:ring-indigo-500 focus-within:ring-offset-2 hover:border-gray-400"),
					h.A(
						h.Href(item.GetViewHref(ctx.Echo())),
						h.Class("grow focus:outline-none"),
						h.Span(
							h.Class("absolute inset-0"),
							h.Aria("hidden", "true")),
						h.P(
							h.Class("text-sm font-medium text-gray-900"),
							g.Raw(item.Title))),
					badges[item.HousingType])
			})))
}

// BrowserGrid displays the provided items in a grid. It does not handle if
// there are no items in the list.
func BrowserGrid(ctx echo.Context, page controller.Page, items []postbase.PostBase, featurePhotos []sqlc.Photo) g.Node {
	return h.Ul(
		h.Role("list"),
		h.Class("grid grid-cols-2 gap-x-4 gap-y-8 sm:grid-cols-3 sm:gap-x-6 lg:grid-cols-4 xl:gap-x-8"),
		g.Group(g.Map(items,
			func(item postbase.PostBase) g.Node {
				featuredPhoto := item.GetFeaturedPhoto(featurePhotos)
				href := item.GetViewHref(ctx.Echo())
				return h.Li(
					h.Class("relative"),
					h.Div(
						h.Class("group aspect-h-1 aspect-w-1 block w-full overflow-hidden rounded-lg bg-gray-100 focus-within:ring-2 focus-within:ring-indigo-500 focus-within:ring-offset-2 focus-within:ring-offset-gray-100"),
						LazyIfElse(featuredPhoto.Ok(),
							func() g.Node {
								unwrapped := featuredPhoto.Unwrap()
								return h.Img(
									h.Class("pointer-events-none object-cover group-hover:opacity-75"),
									h.Src(page.ToUserContentPath(unwrapped.GetThumbnailFilename())))
							},
							IconEmptyPhoto),
						h.A(
							h.Class("absolute inset-0 focus:outline-none"),
							h.Href(href)),
						TextSROnly("View post")),
					h.A(
						h.Class("mt-2 block truncate text-sm font-medium text-gray-900"),
						h.Href(href),
						g.Raw(item.Title)))
			})))
}
