package components

import (
	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"
)

type Stat g.Node

func StatRow(stats ...Stat) g.Node {
	smGridCols := " sm:grid-cols-3"
	lgGridCols := " lg:grid-cols-4"
	switch len(stats) {
	case 1, 2:
		smGridCols = ""
		lgGridCols = ""
	case 3:
		lgGridCols = ""
	case 4:
	default:
		panic("Too many stats for one row.")
	}

	return h.Div(
		h.Class("grid grid-cols-2 gap-px bg-white/5"+smGridCols+lgGridCols),
		g.Map(stats, func(s Stat) g.Node { return g.Node(s) }))
}

type StatSize string

const (
	StatSmall StatSize = "text-lg"
	StatLarge StatSize = "text-2xl"
)

type StatOptions struct {
	Label string
	Value string
	Size  StatSize
}

func NewStat(options StatOptions) Stat {
	if options.Size == "" {
		options.Size = StatLarge
	}

	return h.Div(
		h.Class("py-6"),
		h.P(
			h.Class("text-sm/5 font-medium text-gray-500"),
			g.Text(options.Label)),
		h.P(
			h.Class("mt-3 flex items-baseline gap-x-2"),
			h.Span(
				h.Class("font-semibold tracking-tight text-gray-900 "+string(options.Size)),
				g.Text(options.Value))))
}
