package views

import (
	"database/sql"
	"fmt"
	"io"

	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/config"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
)

type Donate struct {
	CustomerID     sql.NullString
	PublishableKey string
	ReturnURL      string
	Page           controller.Page
}

func (v *Donate) Render(w io.Writer) error {
	content := c.WideContainer(
		c.PrimaryHeader("Donate to OrcasHub"),
		h.Div(
			h.Class("lg:grid lg:grid-cols-2 lg:pt-16 lg:space-x-6 justify-center"),
			h.Div(
				v.DonationForm()),
			h.Div(
				v.PersonalDescription())))

	l := layouts.Primary{
		Content: content,
		Page:    v.Page,
	}
	return l.Render(w)
}

func (v *Donate) PersonalDescription() g.Node {
	return h.Section(
		h.Class("relative pt-16 lg:pt-0 lg:-mt-10"),
		h.Div(
			h.Class("relative mx-auto max-w-5xl pt-16"),
			h.Div(
				h.Class("bg-slate-50 pt-px sm:rounded-3xl"),
				h.Div(
					h.Class("relative mx-auto -mt-16 h-44 w-44 overflow-hidden rounded-full bg-slate-200 md:float-right md:h-64 md:w-64 md:[shape-outside:circle(40%)] lg:mr-20 lg:h-54 lg:w-64 xl:h-72 xl:w-72"),
					h.Img(
						h.Class("absolute inset-0 h-full w-full object-cover"),
						h.Src(fmt.Sprintf("%s/%s", config.StaticPrefix, "tony-grosinger.jpg")),
					)),
				h.Div(
					h.Class("px-4 py-10 sm:px-10 sm:py-16 md:py-20 lg:py-32"),
					h.P(
						h.Class("mt-8 font-display text-4xl font-extrabold tracking-tight text-slate-900 sm:text-5xl"),
						h.Span(
							h.Class("block text-blue-600 py-2"),
							g.Text("Tony Grosinger -")),
						g.Text("Hi, I'm the one behind the scenes at OrcasHub.")),
					h.Div(
						h.Class("pt-4 space-y-4 text-lg tracking-tight text-slate-700"),
						h.P(
							g.Text("I've lived on Orcas since 2018 and I am honored to call it home.")),
						h.P(
							g.Text("Thank you so much for using OrcasHub. I am so grateful for the community that has embraced this project and supported my efforts to build a tool to bring people together.")),
						h.P(
							g.Text("I love working on OrcasHub, but it does cost money and an incredible amount of time to build and operate. "+
								"Your support directly contributes to keeping OrcasHub alive, growing, and serving our community.")),
					)))))
}

func (v *Donate) DonationForm() g.Node {
	// TODO: Enable Stripe Link

	return h.Div(
		h.Class("max-w-lg mx-auto pt-10 lg:pt-0"),
		c.XData("{'amount': '10', 'custom_amount': '', 'custom_amount_selected': false, 'annual': true}"),
		h.Form(
			h.ID("payment-form"),
			h.Div(
				h.ID("error-message-container"),
				h.Class("rounded-md bg-red-50 p-4 hidden"),
				h.Div(
					h.Class("flex"),
					h.Div(
						h.Class("flex-shrink-0"),
						c.IconRedExclamationTriangle()),
					h.Div(
						h.Class("ml-3"),
						h.Div(
							h.Class("mt-2 text-sm text-red-700"),
							h.ID("error-message"))))),
			c.LazyIf(!v.Page.IsAuth,
				func() g.Node {
					return h.Div(
						h.Label(
							h.Class("block text-md font-medium leading-6 text-gray-700"),
							g.Text("Email")),
						c.InputEmailFullWidth(c.FormInputParams{
							Name:      "email",
							Display:   "Email",
							Autofocus: true,
							Required:  true,
						},
							c.XOn("blur", "createCustomer")))
				}),
			h.Div(
				h.Class("py-2 flex space-x-3"),
				h.Div(
					c.XClick("annual = true; toggleElementsMode('subscription');"),
					c.XBind("class",
						`{
							'flex items-center justify-center rounded-md py-3 px-3 text-sm font-semibold sm:flex-1 bg-white cursor-pointer focus:outline-none': true,
							'ring-2 ring-indigo-600': annual,
							'ring-1 ring-inset ring-gray-200 text-gray-900 hover:bg-gray-50': !annual,
						}`),
					h.Span(g.Text("Annual"))),
				h.Div(
					c.XClick("annual = false; toggleElementsMode('payment');"),
					c.XBind("class",
						`{
							'flex items-center justify-center rounded-md py-3 px-3 text-sm font-semibold sm:flex-1 bg-white cursor-pointer focus:outline-none': true,
							'ring-2 ring-indigo-600': !annual,
							'ring-1 ring-inset ring-gray-200 text-gray-900 hover:bg-gray-50': annual,
						}`),
					h.Span(g.Text("One-time"))),
			),
			h.Input(
				h.Type("hidden"),
				h.Name("period"),
				h.ID("period-input"),
				c.XBind("value", "annual ? 'year' : 'one-time'")),
			h.Input(
				h.Type("hidden"),
				h.Name("amount"),
				h.ID("amount-input"),
				c.XBind("value", "custom_amount_selected ? custom_amount : amount")),
			h.Div(
				h.ID("amount-radio"),
				h.Class("py-2"),
				h.Label(
					h.Class("block text-md font-medium leading-6 text-gray-700"),
					g.Text("Donation amount")),
				h.Div(
					h.Class("grid gap-3 grid-cols-3 sm:grid-cols-5"),
					v.donationAmountButton("5"),
					v.donationAmountButton("10"),
					v.donationAmountButton("25"),
					v.customDonationAmountButton())),
			h.Div(
				h.Class("pb-2"),
				h.Label(
					h.Class("block text-md font-medium leading-6 text-gray-700"),
					g.Text("Name")),
				c.InputTextFullWidth(c.FormInputParams{
					Name:     "name",
					Display:  "Name",
					Required: true,
				})),
			h.Div(h.ID("payment-element")),
			h.Button(
				h.ID("submit-button"),
				h.Class("inline-flex items-center justify-center gap-x-2 rounded-md border border-transparent bg-indigo-600 px-3 py-2 mt-4 font-medium leading-4 text-white shadow-sm hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-indigo-500 focus:ring-offset-2 sm:text-sm disabled:bg-gray-200 disabled:cursor-not-allowed disabled:hover:bg-gray-200 disabled:ring-0 disabled:text-gray-400"),
				c.XText("annual ? 'Subscribe' : 'Donate'"),
				g.Text("Subscribe")),
			h.Div(
				h.ID("error-message"))),
		v.Script(),
	)
}

func (v *Donate) donationAmountButton(amount string) g.Node {
	return h.Div(
		c.XClick(fmt.Sprintf("amount = '%s'; custom_amount_selected = false;", amount)),
		c.XBind("class",
			fmt.Sprintf(`{
				'flex items-center justify-center rounded-md py-3 px-3 text-sm font-semibold sm:flex-1 bg-white cursor-pointer focus:outline-none': true,
				'ring-2 ring-indigo-600': !custom_amount_selected && amount === '%s',
				'ring-1 ring-inset ring-gray-200 text-gray-900 hover:bg-gray-50': custom_amount_selected || amount !== '%[1]s',
			}`, amount)),
		h.Span(g.Text("$"+amount)),
		h.Span(
			h.Class("text-gray-500"),
			c.XShow("annual"),
			g.Text("/yr")))
}

func (v *Donate) customDonationAmountButton() g.Node {
	return h.Div(
		c.XClick(fmt.Sprintf("custom_amount_selected = true;")),
		c.XBind("class",
			`{
				'col-span-2 flex items-center justify-center rounded-md py-3 px-3 text-sm font-semibold sm:flex-1 bg-white cursor-pointer focus:outline-none': true,
				'ring-2 ring-indigo-600': custom_amount_selected,
				'ring-1 ring-inset ring-gray-200 text-gray-900 hover:bg-gray-50': !custom_amount_selected,
			}`),
		h.Div(
			h.Class("relative rounded-md shadow-sm"),
			h.Div(
				h.Class("pointer-events-none absolute inset-y-0 left-0 flex items-center pl-3"),
				h.Span(
					h.Class("text-gray-500 sm:text-sm"),
					g.Text("$"))),
			h.Input(
				h.Class("block w-full rounded-md border-0 py-1.5 pl-7 pr-12 text-gray-900 ring-1 ring-inset ring-gray-200 placeholder:text-gray-400 focus:ring-1 focus:ring-inset focus:ring-gray-500 sm:text-sm sm:leading-6"),
				h.Type("number"),
				h.Min("0"),
				h.Step("1"),
				h.Placeholder("Custom"),
				c.XModel("custom_amount")),
			h.Div(
				h.Class("pointer-events-none absolute inset-y-0 right-0 flex items-center pr-3"),
				h.Span(
					h.Class("text-gray-500 sm:text-sm"),
					c.XShow("annual"),
					g.Text("/yr"))),
		))
}

func (v *Donate) Script() g.Node {
	return h.Script(
		g.Rawf(`
const stripe = Stripe('%s');
const csrf = '%s';
let customerID = '%s';

const createCustomer = async (event) => {
	clearError();

	const email = event.target.value;
	if (!email) {
		return;
	}

	if (!email.includes('@') || !email.includes('.')) {
		setError('Please enter a valid email address.');
		return;
	}

	const formData = new FormData();
	formData.append('csrf', csrf);
	const response = await fetch('/api/stripe/customer?email=' + email, {
		method: 'POST',
		body: formData,

	});
	const data = await response.json();
	if (data.status === 'ok') {
		customerID = data.customer_id;
	} else {
		console.error('Error:', data.message);
		setError('There was an error finding your account. Please try again later.')
	}
};

const paymentForm = document.getElementById('payment-form');
paymentForm.addEventListener('submit', async(event) => {
	event.preventDefault();

	const submitButton = document.getElementById('submit-button');
	if (submitButton.disabled) {
		return; // Prevent double submissions
	}
	submitButton.disabled = true;

	const amountInput = document.getElementById('amount-input');
	const amount = amountInput.value;

	const nameInput = document.getElementById('name');
	const name = nameInput.value;

	if (!amount) {
		setError("Please select a payment amount.");
		submitButton.disabled = false;
		return;
	}

	elements.update({
		amount: amount * 100,
	})

	// Trigger form validation and wallet collection
	const {error: submitError} = await elements.submit();
	if (submitError) {
		setError(submitError);
		submitButton.disabled = false;
		return;
	}

	const period = document.getElementById('period-input').value;
	
	if (period === 'year') {
		processSubscription(amount, name);
	} else {
		processOneTime(amount, name);
	}
});

const processOneTime = async (amount, name) => {
	const formData = new FormData();
	formData.append('csrf', csrf);
	formData.append('amount', amount * 100);
	formData.append('customer-id', customerID);
	formData.append('name', name);
	const response = await fetch("/api/stripe/payment-intent", {
		method: "POST",
		body: formData,
	});
	const { clientSecret } = await response.json();

	const {error} = await stripe.confirmPayment({
		elements,
		clientSecret,
		confirmParams: {
			return_url: '%s',
		},
	});

	if (error) {
		// This point is only reached if there's an immediate error when confirming the Intent.
		// Show the error to your customer (for example, "payment details incomplete").
		setError(message);
		submitButton.diabled = false;
		return;
	} else {
		// Your customer is redirected to your return_url. For some payment
		// methods like iDEAL, your customer is redirected to an intermediate
		// site first to authorize the payment, then redirected to the return_url.
	}
};

const processSubscription = async (amount, name) => { 
	const formData = new FormData();
	formData.append('csrf', csrf);
	formData.append('amount', amount * 100);
	formData.append('customer-id', customerID);
	formData.append('name', name);
	const res = await fetch('/api/stripe/subscription', {
		method: "POST",
		body: formData,
	});
	const {status, message, type, clientSecret} = await res.json();
	if (status !== "ok") {
		setError(message);
		submitButton.disabled = false;
		return;
	}

	const confirmIntent = type === "setup" ? stripe.confirmSetup : stripe.confirmPayment;

	// Confirm the Intent using the details collected by the Payment Element
	const {error} = await confirmIntent({
		elements,
		clientSecret,
		confirmParams: {
			return_url: '%[4]s',
		},
	});

	if (error) {
		// This point is only reached if there's an immediate error when confirming the Intent.
		// Show the error to your customer (for example, "payment details incomplete").
		setError(message);
		submitButton.diabled = false;
		return;
	} else {
		// Your customer is redirected to your return_url. For some payment
		// methods like iDEAL, your customer is redirected to an intermediate
		// site first to authorize the payment, then redirected to the return_url.
	}
};

const toggleElementsMode = (mode) => {
	if (mode === 'subscription') {
		elements.update({
			mode: 'subscription',
			setup_future_usage: 'off_session',
		});
	} else {
		elements.update({
			mode: 'payment',
			setup_future_usage: 'on_session',
		});
	}
};

const setError = (message) => {
	if (!(message instanceof String) || message === '[object Object]' || !message) {
		message = "Please resolve the errors below and try again";
	}

	const errorMessageContainerEl = document.getElementById('error-message-container');
	errorMessageContainerEl.classList.remove('hidden');
	const errorMessageEl = document.getElementById('error-message');
	errorMessageEl.textContent = message;
};

const clearError = () => {
	const errorMessageContainerEl = document.getElementById('error-message-container');
	errorMessageContainerEl.classList.add('hidden');
};

const options = {
	mode: 'subscription',
	setup_future_usage: 'off_session',
	amount: 1000,
	currency: 'usd',
	// Fully customizable with appearance API.
	appearance: {/*...*/},
};

// Set up Stripe.js and Elements to use in checkout form, passing the client secret obtained in step 5
const elements = stripe.elements(options);

// Create and mount the Payment Element
const paymentElement = elements.create('payment');
paymentElement.mount('#payment-element');
		`,
			v.PublishableKey,
			v.Page.CSRF,
			v.CustomerID.String,
			v.ReturnURL,
		))
}
