package views

import (
	"io"
	"net/http"

	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
)

type Error struct {
	Page controller.Page
}

func (v *Error) Render(w io.Writer) error {
	content := c.WideContainer(
		c.CenteredX(
			h.Div(
				h.Class("text-center"),
				c.LazySwitch(
					c.LazyCond{
						Cond:   v.Page.StatusCode == http.StatusForbidden || v.Page.StatusCode == http.StatusUnauthorized,
						Result: v.unauthorized,
					},
					c.LazyCond{
						Cond:   v.Page.StatusCode == http.StatusNotFound,
						Result: v.notFound,
					},
					c.LazyCond{
						// Default
						Cond:   true,
						Result: v.internalServerError,
					},
				),
				h.A(
					h.Href(v.Page.ToURL(pages.Names.Home)),
					h.Class("mt-6 block text-base font-medium text-indigo-600 hover:text-indigo-500"),
					g.Text("Go back home"),
					h.Span(
						h.Aria("hidden", "true"),
						g.Raw(" &rarr;"))))),
	)

	l := layouts.Primary{
		Content: content,
		Page:    v.Page,
	}
	return l.Render(w)
}

func (v *Error) internalServerError() g.Node {
	return h.Div(
		h.Class("space-y-2"),
		c.PrimaryHeader("Something went wrong"),
		c.TextNormal("Please try again."))
}

func (v *Error) unauthorized() g.Node {
	return h.Div(
		h.Class("space-y-2"),
		c.PrimaryHeader("Unauthorized"),
		c.TextNormal("You are not authorized to view the requested page."))
}

func (v *Error) notFound() g.Node {
	return h.Div(
		h.Class("space-y-2"),
		c.PrimaryHeader("Page not found"),
		c.TextNormal("Sorry, we couldn't find the page you're looking for."))
}
