package views

import (
	"io"

	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
)

type EventPhotoUpload struct {
	PublishAction string
	EventID       int
	Token         string

	Page controller.Page
}

func (v *EventPhotoUpload) Render(w io.Writer) error {
	content := c.NarrowContainer(
		h.Div(
			c.XData("{}"), // Needed so the ButtonCancelBack works.
			h.Class("flex"),
			h.Div(
				h.Class("grow"),
				c.PrimaryHeader("Add Photos")),
			c.ButtonCancelBack()),
		h.Div(
			h.Class("text-md my-2 max-w-2xl space-y-2 font-medium text-gray-900"),
			c.TextNormal("Events may have only one photo. Uploading a photo will replace any existing photo on this event.")),
		h.Div(
			h.Input(
				h.Class("mt-4"),
				h.Type("file"),
				h.Accept("image/jpeg, image/png")),
			c.TextDescription("PNG or JPG files, 32MB or smaller."),
			h.Div(
				h.Class("w-full max-w-full border-b border-gray-300"),
				h.Div(
					h.ID("progress-bar"),
					h.Class("h-1 bg-indigo-700"),
					h.Style("width: 0%;")))),
		g.Group([]g.Node{
			h.Script(
				h.Src("/files/js/tus-4.2.3.js")),
			v.script(),
		}),
	)

	l := layouts.Primary{
		Content: content,
		Page:    v.Page,
	}
	return l.Render(w)
}

func (v *EventPhotoUpload) script() g.Node {
	// Upload resumption is disabled, but check demo if you want to re-enable.
	return h.Script(
		g.Rawf(`
const event_id = %d;
const token = %q;
const support_email = %q;
let upload = null;
let uploadIsRunning = false;
const input = document.querySelector("input[type=file]");
const progressBar = document.querySelector("#progress-bar");

function startUpload() {
	const file = input.files[0];
	if (!file) {
		return;
	}

	const options = {
		endpoint: "/uploads",
		retryDelays: [0, 1000, 3000, 5000],
		uploadDataDuringCreation: true,
		storeFingerprintForResuming: false,
		metadata: {
			ext: file.name.slice((file.name.lastIndexOf(".") - 1 >>> 0) + 2),
			event_id: event_id,
			token: token
		},
		onError(error) {
			console.error(error);

			if (error.originalRequest) {
				if (window.confirm("Upload failed.\nDo you want to retry?")) {
					upload.start();
					uploadIsRunning = true;
					return;
				}
			} else {
				const message = "Upload failed. Please contact " + support_email + " for assistance"
				window.alert(message);
			}

			reset();
		},
		onProgress(bytesUploaded, bytesTotal) {
			const percentage = ((bytesUploaded / bytesTotal) * 100).toFixed(2);
			progressBar.style.width = percentage + '%%';
			console.log(bytesUploaded, bytesTotal, percentage + '%%');
		},
		onSuccess() {
			input.value = "";
			setTimeout(()=> window.location.replace(%q), 1000);
		},
	};

	upload = new tus.Upload(file, options);
	input.disabled = true;
	upload.start();
	uploadIsRunning = true;
}

input.addEventListener("change", startUpload);`,
			v.EventID,
			v.Token,
			v.Page.Cfg.Mail.SupportAddress,
			v.Page.ToURL(pages.Names.Event.One, v.EventID)))
}
