package helpers

import (
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
)

func TestFormatDuration(t *testing.T) {
	cases := []struct {
		Name     string
		Input    time.Duration
		Expected string
	}{
		{
			Name:     "Less than an hour",
			Input:    5 * time.Minute,
			Expected: "5 minutes",
		},
		{
			Name:     "Less than two hours",
			Input:    100 * time.Minute,
			Expected: "100 minutes",
		},
		{
			Name:     "Less than a day",
			Input:    3 * time.Hour,
			Expected: "3 hours",
		},
		{
			Name:     "An odd number of minutes above an hour",
			Input:    125 * time.Minute,
			Expected: "2 hours",
		},
		{
			Name:     "One day",
			Input:    24 * time.Hour,
			Expected: "1 day",
		},
		{
			Name:     "Slightly more than a day",
			Input:    24*time.Hour + 5*time.Minute,
			Expected: "1 day",
		},
		{
			Name:     "Several days",
			Input:    3*24*time.Hour + 5*time.Hour,
			Expected: "3 days",
		},
	}

	for _, tc := range cases {
		t.Run(tc.Name, func(t *testing.T) {
			result := FormatDuration(tc.Input)
			assert.Equal(t, tc.Expected, result)
		})
	}
}
