package helpers

import (
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestPageNums(t *testing.T) {
	cases := []struct {
		Name        string
		NumPages    int
		CurrentPage int
		Expected    []int
	}{
		{
			Name:        "Only a single page",
			NumPages:    1,
			CurrentPage: 1,
			Expected:    []int{1},
		},
		{
			Name:        "Short range",
			NumPages:    5,
			CurrentPage: 2,
			Expected:    []int{1, 2, 3, 4, 5},
		},
		{
			Name:        "At the start of the range",
			NumPages:    30,
			CurrentPage: 1,
			Expected:    []int{1, 2, 3, 4, 5, 6, 7},
		},
		{
			Name:        "Near the start of the range",
			NumPages:    30,
			CurrentPage: 2,
			Expected:    []int{1, 2, 3, 4, 5, 6, 7},
		},
		{
			Name:        "Middle of the range",
			NumPages:    30,
			CurrentPage: 6,
			Expected:    []int{3, 4, 5, 6, 7, 8, 9},
		},
		{
			Name:        "Near the end of the range",
			NumPages:    30,
			CurrentPage: 28,
			Expected:    []int{24, 25, 26, 27, 28, 29, 30},
		},
		{
			Name:        "End of the range",
			NumPages:    30,
			CurrentPage: 30,
			Expected:    []int{24, 25, 26, 27, 28, 29, 30},
		},
	}

	for _, tc := range cases {
		t.Run(tc.Name, func(t *testing.T) {
			result := PageNums(tc.NumPages, tc.CurrentPage)
			assert.Equal(t, tc.Expected, result)
		})
	}
}
