package views

import (
	"fmt"
	"io"

	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/config"
	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/helpers"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
)

type MagicSigninSent struct {
	EmailAddress string
	Message      string

	Page controller.Page
}

func (v *MagicSigninSent) Render(w io.Writer) error {
	content := c.NarrowContainer(
		h.Div(
			h.Class("text-center text-lg"),
			c.SpacedYLarge(
				c.TertiaryHeader(v.Message),
				h.Div(
					h.ID("checking-message"),
					h.Div(
						h.Class("flex items-end justify-center space-x-2"),
						h.P(
							g.Text("We're sending a message to "),
							h.Span(
								h.ID("email-address"),
								h.Class("font-semibold"),
								g.Text(v.EmailAddress))),
						h.Div(
							h.Class("hidden sm:block"),
							c.IconAnimatedLoadingDots())),
					h.P(
						h.Class("text-base text-gray-500"),
						g.Text("(Please click the link in your email)"))),
				h.Div(
					h.ID("success-message"),
					h.Class("hidden"),
					h.Div(
						h.Class("rounded-md bg-green-50 text-green-800 p-4 space-y-2"),
						h.P(g.Text("Message delivered sucessfully. Please check your email.")))),
				h.Div(
					h.ID("error-message"),
					h.Class("hidden"),
					h.Div(
						h.Class("rounded-md bg-red-50 text-red-800 p-4 space-y-2"),
						h.P(g.Text("We were unable to email you.")),
						h.P(g.Text("Perhaps there was a typo in your email address?")),
						c.ButtonPrimary(
							v.Page.ToURL(pages.Names.User.Login),
							g.Text("Login again")),
						h.P(
							h.Class("text-sm"),
							g.Text("Or "),
							c.TextLink("mailto:"+v.Page.Cfg.Mail.SupportAddress, g.Text("contact support")),
							g.Text(" if this continues occuring.")))),
			)),
		h.Script(
			h.Defer(),
			h.Src(
				fmt.Sprintf("/%s/js/delivery-validation.js?v=%s", config.StaticPrefix, helpers.CacheBuster))),
	)

	l := layouts.Primary{
		Content: content,
		Page:    v.Page,
	}
	return l.Render(w)
}
