package views

import (
	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/constants"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/models/postbase"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/helpers"
)

type MyPostsTable struct {
	Page controller.Page
}

// Table creates the My Posts table for the posts contaned in v.
func (v *MyPostsTable) Table(posts []postbase.PostBase, newPostHref string) g.Node {
	if len(posts) == 0 {
		return h.Div(
			h.Class("py-8 text-center"),
			h.H3(
				h.Class("mt-2 text-sm font-medium text-gray-900"),
				g.Text("No posts")),
			c.TextDescription("You don't have any posts of this type right now.", "mt-1"),
			h.Div(
				h.Class("mt-6"),
				h.A(
					h.Href(newPostHref),
					h.Class("inline-flex items-center rounded-md border border-transparent bg-indigo-600 px-4 py-2 text-sm font-medium text-white shadow-sm hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-indigo-500 focus:ring-offset-2"),
					c.IconSolidPlus(
						h.Class("-ml-1 mr-2 h-5 w-5")),
					g.Text("New Post"))))
	}

	return h.Div(
		h.Class("-mx-4 mt-8 overflow-hidden shadow ring-1 ring-black ring-opacity-5 sm:-mx-6 md:mx-0 md:rounded-lg"),
		c.Table(
			[]g.Node{
				c.TableHeaderCol("Title", "", false),
				c.TableHeaderCol("Status", "hidden sm:table-cell", false),
				c.TableHeaderCol("Expiration", "hidden lg:table-cell", false),
				c.TableHeaderCol("Actions", "", true),
			},
			g.Map(posts, v.postRow),
		))
}

func (v *MyPostsTable) postRow(post postbase.PostBase) g.Node {
	echo := v.Page.Context.Echo()
	status := post.GetStatus()
	var expiration string
	var statusText string
	var statusPill g.Node
	var mobileStatusButton g.Node
	switch status {
	case constants.Draft:
		statusText = "Draft"
		statusPill = c.TextPill("Draft", "bg-indigo-10 text-indigo-800")
		mobileStatusButton = c.TextButtonLink(post.GetAddPhotosHref(echo), "Publish")
	case constants.Expired:
		expiration = "Expired"
		statusText = "Expired"
		statusPill = c.TextPill("Expired", "bg-gray-10 text-gray-800")
		mobileStatusButton = c.TextButtonLink(post.GetRelistHref(echo), "Relist")
	case constants.Unlisted:
		expiration = "Inactive"
		statusText = "Inactive"
		statusPill = c.TextPill("Inactive", "bg-gray-10 text-gray-800")
		mobileStatusButton = c.TextButtonLink(post.GetRelistHref(echo), "Relist")
	case constants.Active:
		expiration = helpers.RelativeDateTime(post.GetExpiration())
		statusText = "Expires in " + expiration
		statusPill = c.TextPill("Active", "bg-green-10 text-green-800")
		mobileStatusButton = c.TextButtonLink(post.GetDelistHref(echo), "Delist")
	}

	actionButtons := h.Span(
		h.Class("ml-4 flex flex-shrink-0 justify-end space-x-4"),
		mobileStatusButton,
		h.Span(
			h.Class("text-gray-300"),
			h.Aria("hidden", "true"),
			g.Text("|")),
		c.TextButtonLink(post.GetDeleteHref(echo), "Delete"))

	return h.Tr(
		h.Td(
			h.Class("w-full max-w-0 py-4 pl-4 pr-3 text-sm font-medium text-gray-900 sm:w-auto sm:max-w-none sm:pl-6"),
			h.A(
				h.Class("hover:text-indigo-900 hover:underline"),
				h.Href(post.GetViewHref(v.Page.Context.Echo())),
				g.Raw(post.Title)),
			h.Div(
				h.Class("flex font-normal lg:hidden"),
				h.Div(
					h.Class("mt-1 truncate text-gray-700"),
					g.Text(statusText)),
				h.Div(
					h.Class("mt-1 truncate text-gray-500 sm:hidden"),
					actionButtons))),
		h.Td(
			h.Class("px-3 py-4 text-sm text-gray-500"),
			statusPill),
		h.Td(
			h.Class("hidden px-3 py-4 text-sm text-gray-500 lg:table-cell"),
			g.Text(expiration)),
		h.Td(
			h.Class("hidden py-4 pl-3 pr-4 text-right text-sm font-medium sm:table-cell sm:pr-6"),
			actionButtons))
}
