package views

import (
	"io"
	"net/http"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
)

type PostModeratorDeleteForm struct {
	Reason        string `form:"reason" valid:"required"`
	CustomMessage string `form:"custom_message" valid:"optional"`
	Submission    controller.FormSubmission
}

type PostModeratorDelete struct {
	Form PostModeratorDeleteForm
	Page controller.Page
}

func (v *PostModeratorDelete) Render(w io.Writer) error {
	content := c.NarrowContainer(
		c.FullScreenForm(
			http.MethodPost,
			v.Page.Path,
			"",
			"",
			c.FormFooter(nil, "back", "Delete post", true),
			c.FormSection(
				c.FormHeading(
					"Delete post",
					"Are you sure you want to delete this post? This action cannot be undone."),
				c.CSRF(v.Page),
				c.InputText(c.FormInputParams{
					Name:     "reason",
					Display:  "Reason",
					Required: true,
					Value:    v.Form.Reason,
					Errors:   v.Form.Submission.GetFieldErrors("Reason"),
				}),
				c.InputTextArea(c.FormInputParams{
					Name:     "custom_message",
					Display:  "Custom Message",
					Hint:     "If provided, this message will be sent to the user instead of the standard Terms of Service violation notice. Special characters are not allowed.",
					Required: false,
					Value:    v.Form.CustomMessage,
					Errors:   v.Form.Submission.GetFieldErrors("CustomMessage"),
				}))),
	)

	l := layouts.Primary{
		Content: content,
		Page:    v.Page,
	}
	return l.Render(w)
}
