package views

import (
	"io"
	"net/http"

	g "maragu.dev/gomponents"
	h "maragu.dev/gomponents/html"

	"git.grosinger.net/tgrosinger/tonyslist/constants/pages"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
)

// RegisterForm is used for both the signin and register process. They are
// performed from the same page.
type RegisterForm struct {
	Email string `form:"email" valid:"required,email"`

	Action string `form:"action" valid:"optional"`

	// Empty is a field which must be empty when the form is submitted. If it's
	// filled, the form was likely submitted by a bot. This field is not
	// displayed to users.
	Empty string `form:"maiden_name" valid:"optional"`

	Submission controller.FormSubmission
}

type Register struct {
	Form RegisterForm
	Page controller.Page
}

func (v *Register) Render(w io.Writer) error {
	content := h.Div(
		h.Class("flex min-h-full flex-col justify-center py-12 sm:px-6 lg:px-8"),
		h.Div(
			h.Class("mt-8 sm:mx-auto sm:w-full sm:max-w-md"),
			h.Div(
				h.Class("bg-white px-4 shadow sm:rounded-lg sm:px-10"),
				h.Div(
					h.Class("text-center"),
					h.H1(
						h.Class("py-4 text-3xl text-gray-700"),
						g.Text("Sign in to OrcasHub"))),
				h.Form(
					h.Class("py-4"),
					h.Method(http.MethodPost),
					c.CSRF(v.Page),
					c.InputEmailFullWidth(c.FormInputParams{
						Name:      "email",
						Display:   "Email Address",
						Required:  true,
						Autofocus: true,
						Value:     v.Form.Email,
						Errors:    v.Form.Submission.GetFieldErrors("Email"),
					}),
					c.MothersMaidenName(),
					v.Terms("Register or Login"),
					h.Div(
						h.Class("flex gap-x-4"),
						h.Div(
							h.Class("w-1/2 grow"),
							v.Button("Register", "action", "register"),
						),
						h.Div(
							h.Class("w-1/2 grow"),
							v.Button("Login", "action", "login"),
						),
					)))),
	)

	l := layouts.Primary{
		Content: content,
		Page:    v.Page,
	}
	return l.Render(w)
}

func (v *Register) Terms(buttonText string) g.Node {
	return h.P(
		h.Class("pt-6 text-xs text-gray-500"),
		g.Textf("Clicking %s means that you agree to the ", buttonText),
		c.TextLink(
			v.Page.ToURL(pages.Names.TermsOfService),
			g.Text("Terms of Use")),
		g.Text(" and "),
		c.TextLink(
			v.Page.ToURL(pages.Names.PrivacyPolicy),
			g.Text("Privacy Statement")),
		g.Text("."))
}

func (v *Register) Button(display, name, value string) g.Node {
	return h.Div(
		h.Class("mt-2"),
		h.Button(
			h.Type("submit"),
			c.LazyIf(name != "", func() g.Node {
				return h.Name(name)
			}),
			c.LazyIf(value != "", func() g.Node {
				return h.Value(value)
			}),
			h.Class("flex w-full justify-center rounded-md border border-transparent bg-indigo-600 py-2 px-4 text-sm font-medium text-white shadow-sm hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-indigo-500 focus:ring-offset-2"),
			g.Text(display)))
}
