package views

import (
	"io"
	"time"

	g "maragu.dev/gomponents"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/services"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/components/rss"
)

type RSSItem struct {
	Title         string
	PublishedDate time.Time
	Href          string
	Description   string
}

type RSS struct {
	HomeHref     string
	SupportEmail string

	Items []RSSItem

	Page      controller.Page
	Sanitizer *services.Sanitizer
}

func (v *RSS) Render(w io.Writer) error {
	return rss.RSS(
		rss.Channel(
			rss.Title(v.Page.Title),
			rss.Link(v.HomeHref),
			rss.Description(v.Page.Metatags.Description),
			rss.WebMaster(v.SupportEmail),
			g.Group(
				g.Map(v.Items, v.item)),
		),
	).Render(w)
}

func (v *RSS) item(i RSSItem) g.Node {
	return rss.Item(
		rss.Title(i.Title),
		rss.PubDate(i.PublishedDate.Format(time.RFC822)),
		rss.Link(i.Href),
		rss.Description(v.Sanitizer.SanitizeRSS(i.Description)))
}
