package views

import (
	"io"
	"net/http"

	"git.grosinger.net/tgrosinger/tonyslist/pkg/controller"
	c "git.grosinger.net/tgrosinger/tonyslist/pkg/views/components"
	"git.grosinger.net/tgrosinger/tonyslist/pkg/views/layouts"
	g "maragu.dev/gomponents"
)

type RequireNameForm struct {
	Name     string `form:"name"`
	Username string `form:"username"`

	Submission controller.FormSubmission
}

type RequireName struct {
	NameRequired     bool
	UsernameRequired bool

	Form RequireNameForm
	Page controller.Page
}

func (v *RequireName) Render(w io.Writer) error {
	content := c.NarrowContainer(
		c.FullScreenForm(
			http.MethodPost,
			v.Page.URL, // Use URL to preserve the query params.
			"",
			"",
			c.FormFooter(nil, "", "Continue", false),
			c.FormSection(
				c.FormHeading("Information required", "Please provide your name and username to continue."),
				c.CSRF(v.Page),
				c.LazyIf(v.NameRequired, func() g.Node {
					return c.InputText(c.FormInputParams{
						Name:      "name",
						Display:   "Full name",
						Required:  true,
						Autofocus: true,
						Value:     v.Form.Name,
						Errors:    v.Form.Submission.GetFieldErrors("Name"),
					})
				}),
				c.LazyIf(v.UsernameRequired, func() g.Node {
					return c.InputText(c.FormInputParams{
						Name:      "username",
						Display:   "Username",
						Required:  true,
						Autofocus: true,
						Value:     v.Form.Username,
						Errors:    v.Form.Submission.GetFieldErrors("Username"),
					})
				}),
			)))

	l := layouts.Primary{
		Content: content,
		Page:    v.Page,
	}
	return l.Render(w)
}
