function createStyledWhiteSpaceNoWrapSpan(content) {
    const tempSpanElement = document.createElement("span");

    tempSpanElement.style.whiteSpace = "nowrap";
    tempSpanElement.className = "temp";
    tempSpanElement.innerHTML = content;

    return tempSpanElement;
}

function getContentWidth(pre_space, contentString, appendElement) {
    const pre = pre_space ? "&nbsp;" : "";
    const tempSpanElement = createStyledWhiteSpaceNoWrapSpan(pre + contentString);

    appendElement.appendChild(tempSpanElement);

    const width = tempSpanElement.getBoundingClientRect().width;

    appendElement.removeChild(tempSpanElement);

    return width;
}

function calculateMaxLines(element) {
    const containerHeight = element.getBoundingClientRect().height;
    const lineHeight = parseInt(window.getComputedStyle(element).lineHeight);
    return Math.floor(containerHeight / lineHeight);
}

function applyMultilineEllipsisWithMaxLines(lines, max_lines, max_line_width) {
    const end_index = lines.length - 1;
    const line_index = max_lines - 1;

    if (lines.length > max_lines) {
        lines[line_index] =
            '<span style="white-space: nowrap; overflow: hidden; text-overflow: ellipsis; width: ' +
            parseInt(max_line_width) +
            'px; display: inline-block;">' +
            lines[line_index];
        lines[end_index] += "</span>";
    }
}

function calculateLinesContentWordByWordToFitIntoWidth(text, element, max_lines) {
    const paragraphs = text.split(/\n/);
    let lines = [];
    let line_width = 0;
    let current_line = "";

    paragraphs.forEach(function (paragraph, pIndex) {
        if (lines.length > max_lines) {
            return;
        }

        if (!paragraph) {
            lines.push("<br />");
            return;
        }

        const words = paragraph.split(" ");
        words.forEach(function (word, index) {
            if (lines.length > max_lines) {
                lines[lines.length - 1] += ' ' + word;
                return;
            }

            if (line_width == 0) {
                line_width += getContentWidth(false, word, element);
            } else {
                line_width += getContentWidth(true, word, element);
            }

            if (line_width >= element.getBoundingClientRect().width) {
                lines.push(current_line);
                line_width = getContentWidth(false, word, element); // new line
                current_line = "";
            }
            current_line += (current_line != "" ? " " : "") + word;

            if (index == words.length - 1) {
                lines.push(current_line);
            }
        });

        lines.push("<br />");
        current_line = "";
        line_width = 0;
    });

    return lines;
}

function thellipsis(element) {
    const max_lines = calculateMaxLines(element);
    const text = element.innerText;
    const lines = calculateLinesContentWordByWordToFitIntoWidth(text, element, max_lines);

    applyMultilineEllipsisWithMaxLines(lines, max_lines, element.offsetWidth);

    element.innerHTML = lines.join(" ");
}
