# Privacy Statement

Last Updated: 2023-01-15

Your privacy is important to us. This privacy statement explains our collection,
use, and disclosure of personal data. This privacy statement applies to OrcasHub
and to our controlled affiliates and subsidiaries ("OrcasHub"). References to
our "services" in this statement include our websites, apps, devices, and any
other services.

## Personal Data We Collect

The personal data we collect depends on how you interact with us, the services
you use, and the choices you make.

We collect information about you from different sources and in various ways when
you use our services, including information you provide directly, information
collected automatically, information from third-party data sources, and data we
infer or generate from other data.

<span class="underline">Information you provide directly.</span> We collect
personal data you provide to us. For example:

- **Name and contact information.** We collect name, username, or alias, and
  contact details such as email address, postal address, and phone number.
- **Demographic data.** In some cases, such as when you register or participate
  in surveys, we request that you provide age, gender, marital status, and similar
  demographic details.
- **Payment information.** If you make a purchase or other financial
  transaction, we collect credit card numbers, financial account information, and
  other payment details.
- **Content and files.** We collect the photos, documents, or other files you
  upload to our services; and if you send us email messages or other
  communications, we collect and may retain those communications.
- **Sensitive Personal Information.**

**Account access information.** We collect information such as a username or
account number in combination with a password, security or access code, or
other credential that allows access to an account.

<span class="underline">Information we collect automatically.</span> When you
use our services, we collect some information automatically. For example:

- **Identifiers and device information.** When you visit our websites, our web
  servers automatically log your Internet Protocol (IP) address and information
  about your device, including device identifiers (such as MAC address); device
  type; and your device's operating system, browser, and other software including
  type, version, language, settings, and configuration. As further described in
  the [Cookies, Mobile IDs, and Similar Technologies](#cookies-and-similar-technologies)
  section below, our websites and online services store and retrieve cookie
  identifiers and other data.
- **Demographic data.** In some cases, such as when you register or participate
  in surveys, we request that you provide age, gender, marital status, and similar
  demographic details.
- **Usage Data.** We automatically log your activity on our websites, apps and
  connected products, including the URL of the website from which you came to our
  sites, pages you viewed, access times, and other details about your use of and
  actions on our website.

<span class="underline">Information we create or generate.</span> We infer new
information from other data we collect, including using automated means to
generate information about your likely preferences or other characteristics
(**"inferences"**). For example, we infer your general geographic location (such
as city, state, and country) based on your IP address.

<span class="underline">Information we obtain from third-party sources.</span>
We also obtain the types of information described above from third parties.
These third-party sources include, for example:

- **Third-party partners.** Third-party applications and services you choose to
  connect with or interact with through our services.
- **Co-branding/marketing partners.** Partners with which we offer co-branded
  services or engage in joint marketing activities.
- **Service providers.** Third parties that collect or provide data in
  connection with work they do on our behalf, for example companies that process
  payments or determine your device’s location based on its IP address.
- **Publicly availale sources.** Public sources of information such as open
  government databases.

When you are asked to provide personal data, you may decline. And you may use
web browser or operating system controls to prevent certain types of automatic
data collection. But if you choose not to provide or allow information that is
necessary for certain services or features, those services or features may not
be available or fully functional.

## Cookies and Similar Technologies

We use cookies and similar technologies to operate our websites and online
services and to help collect data, including usage data, identifiers, and device
information.

<p class="pb-4 underline">What are cookies and similar technologies?</p>

Cookies are small text files placed by a website and stored by your browser on
your device. A cookie can later be read when your browser connects to a web
server in the same domain that placed the cookie. The text in a cookie contains
a string of numbers and letters that may uniquely identify your device and can
contain other information as well. This allows the web server to recognize your
browser over time, each time it connects to that web server.

<p class="pb-4 underline">
    How do we and our partners use cookies and similar technologies?
</p>

We use these technologies in our websites, apps, and online services to collect
personal data (such as the pages you visit, the links you click on, and similar
usage information, identifiers, and device information) when you use our
services. This data is used to store your preferences and settings, enable you
to sign-in, analyze how our websites and apps perform, track your interaction
with the site or app, develop inferences, combat fraud, and fulfill other
legitimate purposes. We and/or our partners also share the data we collect or
infer with third parties for these purposes. For more information about the
third-party analytics partners that collect personal information on our
services, please see the [Our Disclosure of Personal
Data](#our-disclosure-of-personal-data) section of this statement.

<p class="pb-4 underline">What controls are available?</p>

There are a range of cookie and related controls available through browsers,
mobile operating systems, and elsewhere. See the [Choice and Control of Personal
Data](#choice-and-control-of-personal-data) section below for details.

## Our Use of Personal Data

We use the personal data we collect for purposes described in this privacy
statement or as otherwise disclosed to you. For example, we use personal data
for the following purposes:

- **Produce and service delivery.** To provide and deliver our services,
  including troubleshooting, improving, and personalizing those services.
- **Business operations.** To operate our business, such as billing, accounting,
  improving our internal operations, securing our systems, detecting fraudulent or
  illegal activity, and meeting our legal obligations.
- **Product improvement, development, and research.** To develop new services or
  features, and conduct research.
- **Personalization.** To understand you and your preferences to enhance your
  experience and enjoyment using our services.
- **Customer support.** To provide customer support and respond to your
  questions.
- **Communications.** To send you information, including confirmations,
  invoices, technical notices, updates, security alerts, and support and
  administrative messages.
- **Marketing.** To communicate with you about new services, offers, promotions,
  rewards, contests, upcoming events, and other information about our services and
  those of our selected partners (see the [Choice and
  Control](#choice-and-control-of-personal-data) section of this statement for
  information about how to change your preferences for promotional
  communications).

We combine data we collect from different sources for these purposes to give you
a more seamless, consistent, and personalized experience.

## Our Disclosure of Personal Data

We disclose personal data with your consent or as we determine necessary to
complete your transactions or provide the services you have requested or
authorized. In addition, we disclose each of the categories of personal data
described above, to the types of third parties described below, for the
following business purposes:

- **Public information.** You may select options available through our services
  to publicly display and disclose your name and/or username and certain other
  information, such as your profile, demographic data, content and files, or
  geolocation data.
- **Service providers.** We provide personal data to vendors or agents working
  on our behalf for the purposes described in this statement. For example,
  companies we’ve hired to provide customer service support or assist in
  protecting and securing our systems and services may need access to personal
  data to provide those functions.
- **Financial services and payment processing.** When you provide payment data,
  for example to make a purchase, we will disclose payment and transactional data
  to banks and other entities as necessary for payment processing, fraud
  prevention, credit risk reduction, analytics, or other related financial
  services.
- **Affiliates.** We enable access to personal data across our subsidiaries,
  affiliates, and related companies, for example, where we share common data
  systems or where access helps us to provide our services and operate our
  business.
- **Legal and law enforcement.** We will access, disclose, and preserve personal
  data when we believe that doing so is necessary to comply with applicable law or
  respond to valid legal process, including from law enforcement or other
  government agencies.
- **Security, safety, and protecting rights.** We will disclose personal data if
  we believe it is necessary to: protect our customers and others, for example to
  prevent spam or attempts to commit fraud, or to help prevent the loss of life or
  serious injury of anyone; operate and maintain the security of our services,
  including to prevent or stop an attack on our computer systems or networks; or
  protect the rights or property of ourselves or others, including enforcing our
  agreements, terms, and policies.

Finally, we may disclose de-identified information in accordance with applicable
law.

## Choice and Control of Personal Data

We provide a variety of ways for you to control the personal data we hold about
you, including choices about how we use that data. In some jurisdictions, these
controls and choices may be enforceable as rights under applicable law.

<span class="underline">Access, portability, correction, and deletion.</span> If
you wish to access, correct, or delete personal data about you that we hold, you
may access your account by logging into the OrcasHub services you use.

If you are unable to access, copy, correct, or delete certain personal data we
have via the means described above, you can send us a request by using contact
methods described at the bottom of this privacy statement.

<span class="underline">Communication preferences.</span> You can choose whether
to receive (a) promotional communications and (b) post push notifications from
us by email and physical mail. If you receive promotional email messages or post
push notifications from us and would like to stop, you can do so by (a)
following the directions in that message, (b) contacting us as described in the
“Contact Us” section below, or (c) by using your account settings to control
post push notifications. These choices do not apply to certain informational
communications including surveys and mandatory service communications.

These choices are specific to the device or browser you are using. If you access
our services from other devices or browsers, take these actions from those
systems to ensure your choices apply to the data collected when you use those
systems.

## Retention of Personal Data

We retain personal data for as long as necessary to provide the services and
fulfill the transactions you have requested, comply with our legal obligations,
resolve disputes, enforce our agreements, and other legitimate and lawful
business purposes. Because these needs can vary for different data types in the
context of different services, actual retention periods can vary significantly
based on criteria such as user expectations or consent, the sensitivity of the
data, the availability of automated controls that enable users to delete data,
and our legal or contractual obligations. For example:

- Expired posts will be retained for 4 weeks to facilitate easier reposting.
  After that period the post and associated images will be completely deleted.
- In a future mailing list service, posts by a user may be anonymized but remain
  otherwise intact if a user deletes their account.
- Server logs containing anonymous user access information will be cleaned
  periodically, but likely not on a set schedule, at least for now.

## Location of Personal Data

The personal data we collect may be stored and processed in your country or
region. Currently, we singularly use data storage on Orcas Island. The storage
location is chosen to operate efficiently and improve performance. We take
steps with the intent of processing and protecting as described in this
statement wherever the data are located.

## Security of Personal Data

We take reasonable and appropriate steps to help protect personal data from
unauthorized access, use, disclosure, alteration, and destruction.

To help us protect personal data, we request that you use a strong password and
never share your password with anyone or use the same password with other sites
or accounts.

## Changes to This Privacy Statement

We will update this privacy statement when necessary to reflect changes in our
services, how we use personal data, or the applicable law. When we post changes
to the statement, we will revise the "Last Updated" date at the top of the
statement. If we make material changes to the statement, we will provide notice
or obtain consent regarding such changes as may be required by law.

## How to Contact Us

If you have a privacy concern, complaint, or a question for OrcasHub, please
contact us at [support@orcashub.org](mailto:support@orcashub.org).
